// digidoccom.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "stdio.h"
#include "comutil.h"
#include "libdigidoccom\DigiDocLibCOM_h.h"
#include "config.h"
#include <stdlib.h>
#include <string.h>
//#include <openssl/objects.h>
#include <sys/stat.h>
#include <time.h>
#include <mbstring.h>
#include <atlbase.h>
#include <atlconv.h>
#include <libdigidoc/DigiDocConvert.h>

//==========< global constants >====================
char* p_szInFile = 0;
char* p_szOutFile = 0;
char* g_szProgNameVer = "digidoccom|"DIGIDOC_VERSION;
boolean g_bDdocExists = FALSE;
CLSID g_clsid;
IDispatch *g_pDis = NULL;
IComDigiDocLib *g_pLib = NULL;
IComSignedDoc *g_pSigDoc = NULL;

//==========< helper functions for argument handling >====================

//--------------------------------------------------
// Converts input data to UTF-8
// src - input data
// returns converted string or NULL. Caller must free it.
//--------------------------------------------------
int convertInput(const char* src, char** dest)
{
  int l1, err = 0;

  *dest = 0;
  l1 = strlen(src) * 2 + 3;
  *dest = (char*)malloc(l1);
  if(*dest) {
    memset(*dest, 0, l1);
    err = oem2utf8(src, *dest, l1);
    //strncpy(*dest, src, l1);
  }
  return err;
}

//--------------------------------------------------
// Handles one argument
//--------------------------------------------------
int checkArguments(int argc, _TCHAR* argv[], int* pCounter, char** dest)
{
  int nLen = 0;

  CW2A pszA(argv[(*pCounter)+1]);
  if(argc > (*pCounter)+1 && pszA[0] != '-') {
    nLen = strlen(pszA) * 2;
    convertInput(pszA, dest);
    (*pCounter)++;
  }
  return nLen;
}

//--------------------------------------------------
// checks the existence of one command line argument
//--------------------------------------------------
int hasCmdLineArg(int argc, _TCHAR* argv[], const char* argName)
{
  int i;

  for(i = 1; i < argc; i++) {
	CW2A pszA(argv[i]);
    if(pszA != NULL &&  pszA[0] == '-' &&
       !strcmp(pszA, argName)) {
      return i;
    }
  }
  return 0;
}

//--------------------------------------------------
// Returns the value of one command line argument
//--------------------------------------------------
int checkCmdLineArg(int argc, _TCHAR* argv[], const char* argName, char** dest)
{
  int i;

  *dest = 0; // mark as not found
  for(i = 1; i < argc; i++) {
	CW2A pszA(argv[i]);
    if(pszA != NULL && pszA[0] == '-' &&
       !strcmp(pszA, argName)) {
       if(i + 1 < argc) {
		 CW2A pszB(argv[i+1]);
         if(pszB[0] != '-') {
         convertInput(pszB, dest);
         return 0;
		 }
       }
       else
         return -1;
    }
  }
  return 0; // not found but no error
}

//--------------------------------------------------
// prints usage statement
//--------------------------------------------------
void printUsage()
{
  fprintf(stderr, "USAGE: digidoccom <command(s)> [-in <input-file>] [-out <output-file>]\n");
  fprintf(stderr, "COMMANDS:\n");
  fprintf(stderr, "\t[-new] [format] [version]\n");
  //fprintf(stderr, "\t-check-cert <-cerificate-file-name (PEM format)>\n");
  fprintf(stderr, "\t-add <file-name> <mime-type> [<content-type>]\n");
  fprintf(stderr, "\t-verify \n");
  fprintf(stderr, "\t-list \n");
  fprintf(stderr, "\t-sign [<pin2> [[[<manifest>] [<city> <state> <zip> <country>]]]\n");
  fprintf(stderr, "\t-extract <doc-id> <file-name>\n");
  fprintf(stderr, "\t-list-certs \n");
  /*fprintf(stderr, "\t-encrypt <file-name>\n");
  fprintf(stderr, "\t-encrypt-sk <file-name>\n");
  fprintf(stderr, "\t-encrecv <cert-file> [<recipient>] [<keyname>] [<carried-key-name>]\n");
  fprintf(stderr, "\t-decrypt <file-name> <pin1> [pkcs12-file] [slot(0)]\n");
  fprintf(stderr, "\t-decrypt-sk <file-name> <pin1> [pkcs12-file] [slot(0)]\n");
  fprintf(stderr, "\t-denc-list <file-name>\n");
  fprintf(stderr, "\t-encrypt-file <in-file-name> <out-file-name> [<in-file-mime-type>]\n");
  fprintf(stderr, "\t-decrypt-file <in-file-name> <out-file-name> <pin1> [pkcs12-file]\n");
  */
}

//--------------------------------------------------
// Checks program runtime arguments
//--------------------------------------------------
int checkProgArguments(int argc, _TCHAR* argv[])
{
  int err = 0;

  // -?, -help -> print usage
  if(!err && (hasCmdLineArg(argc, argv, "-?") ||
     hasCmdLineArg(argc, argv, "-help")))
     printUsage();
  // -in <input-file>
  if(!err)
    if((err = checkCmdLineArg(argc, argv, "-in", &p_szInFile)) != 0) {
      printf("Missing or invalid input file name\n");
	  err = -1;
	}
  // -out <outut-file>
  if(!err)
    if((err = checkCmdLineArg(argc, argv, "-out", &p_szOutFile)) != 0) {
      printf("Missing or invalid output file name\n");
	  err = -1;
	}
  // -encrypt <encryption-input-file>
  /*if(!err && hasCmdLineArg(argc, argv, "-encrypt"))
    if((err = checkCmdLineArg(argc, argv, "-encrypt", &p_szInEncFile)) != ERR_OK)
      addError(err, __FILE__, __LINE__, "Missing or invalid encrypt input file name");
  // -encrypt-sk <encryption-input-file>
  if(!err && hasCmdLineArg(argc, argv, "-encrypt-sk"))
    if((err = checkCmdLineArg(argc, argv, "-encrypt-sk", &p_szInEncFile)) != ERR_OK)
      addError(err, __FILE__, __LINE__, "Missing or invalid encrypt input file name");
  // -config <config-file>
  if(!err)
    if((err = checkCmdLineArg(argc, argv, "-config", &p_szConfigFile)) != ERR_OK)
      addError(err, __FILE__, __LINE__, "Missing or invalid configuration file name");*/
  return err;
}

//==========< COM handlers >====================

int comConnect()
{
	HRESULT hr = CoInitialize(NULL); 
	if(FAILED(hr)) {
        printf("Failed to init com system  %d\n", hr);
		return -1;
    }
	hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComDigiDocLib"), &g_clsid);
	//printf("clsid hr %d\n", hr);
    if(FAILED(hr)) {
        printf("Failed to find clsid of digidoccom  %d\n", hr);
		return -1;
    } 
	hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComDigiDocLib), (LPVOID *)&g_pLib);
    if(FAILED(hr)) {
        printf("Failed to connect to CComDigiDocLib  %d\n", hr);
		return -1;
    } 
	hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComSignedDoc"), &g_clsid);
	if(FAILED(hr)) {
        printf("Failed to find clsid of comsigneddoc  %d\n", hr);
		return -1;
    }
	hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComSignedDoc), (LPVOID *)&g_pSigDoc);
    if(FAILED(hr)) {
        printf("Failed to connect to CComSignedDoc  %d\n", hr);
		return -1;
    }
	return 0;
}

int comDisconnect()
{
	// cleanup
	if(g_pSigDoc)
		g_pSigDoc->Release();
	if(g_pLib)
		g_pLib->Release();
	CoUninitialize();
	return 0;
}

int comGetLibName(char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR libName;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = g_pLib->getLibName(&libName);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(libName);
  ::SysFreeString(libName);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetLibVer(char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR libVer;
  char *p1 = 0, *p2 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = g_pLib->getLibVersion(&libVer);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(libVer);
  ::SysFreeString(libVer);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetComVer(char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR comVer;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = g_pLib->getComVersion(&comVer);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(comVer);
  ::SysFreeString(comVer);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comSetClientNameVer(char* pBuf)
{
  HRESULT hr = 0;
  BSTR bstr = _com_util::ConvertStringToBSTR(pBuf);
  hr = g_pLib->SetClientNameVersion(bstr);
  ::SysFreeString(bstr);
  if(FAILED(hr)) return -2;
  return 0;
}

int comSetDebugFilePath(char* pBuf)
{
  HRESULT hr = 0;
  BSTR bstr = _com_util::ConvertStringToBSTR(pBuf);
  hr = g_pLib->put_DebugFilePath(bstr);
  ::SysFreeString(bstr);
  if(FAILED(hr)) return -2;
  return 0;
}

int comSetDebugLevel(long l)
{
  HRESULT hr = 0;
  hr = g_pLib->put_DebugLevel(l);
  if(FAILED(hr)) return -2;
  return 0;
}

int comReadSignedDoc(char* szFileName)
{
  HRESULT hr = 0;
  long lErr = 0;
  BSTR fname = _com_util::ConvertStringToBSTR(szFileName);
  hr = g_pSigDoc->readSignedDoc(fname, 0, 0, &lErr);
  ::SysFreeString(fname);
  if(FAILED(hr)) return -2;
  return (int)lErr;
}

int comWriteSignedDoc(char* szOldFile, char* szNewFile)
{
  HRESULT hr = 0;
  long lErr = 0;
  BSTR oname = _com_util::ConvertStringToBSTR(szOldFile);
  BSTR nname = _com_util::ConvertStringToBSTR(szNewFile);
  //printf("Writing %s - %s sigdoc: %ld\n", szOldFile, szNewFile, g_pSigDoc);
  hr = g_pSigDoc->createSignedDoc(nname, oname, &lErr);
  //printf("Wrote\n");
  ::SysFreeString(oname);
  ::SysFreeString(nname);
  if(FAILED(hr)) return -2;
  return (int)lErr;
}

int comExtractDataFile(char* szFileName, char* sDocId, char* szDataFileName)
{
  HRESULT hr = 0;
  long lErr = 0;
  BSTR fname = _com_util::ConvertStringToBSTR(szFileName);
  BSTR docId = _com_util::ConvertStringToBSTR(sDocId);
  BSTR docName = _com_util::ConvertStringToBSTR(szDataFileName);
  BSTR charset = _com_util::ConvertStringToBSTR("UTF-8");
  hr = g_pLib->extractDataFile(g_pSigDoc, fname, docName, docId,charset,&lErr);
  ::SysFreeString(fname);
  ::SysFreeString(docId);
  ::SysFreeString(docName);
  ::SysFreeString(charset);
  if(FAILED(hr)) return -2;
  return (int)lErr;
}

int comNewSignedDoc(char* szFormat, char* szVer)
{
  HRESULT hr = 0;
  long lErr = 0;
  BSTR format = _com_util::ConvertStringToBSTR(szFormat);
  BSTR ver = _com_util::ConvertStringToBSTR(szVer);
  hr = g_pSigDoc->initialize(format, ver);
  ::SysFreeString(format);
  ::SysFreeString(ver);
  if(FAILED(hr)) return -2;
  return (int)lErr;
}

int comGetSigDocFormat(char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = g_pSigDoc->get_szFormat(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetSigDocVer(char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = g_pSigDoc->get_szFormatVer(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetErrors(char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = g_pLib->CollectErrors();
  hr = g_pLib->DigiDocErrors(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comCleanErrors()
{
  HRESULT hr = 0;
  hr = g_pLib->ClearDigiDocErrors();
  return (int)hr;
}

int comVerifySignedDoc(char* szDataFileName)
{
  HRESULT hr = 0;
  long lErr = 0;
  BSTR fname = _com_util::ConvertStringToBSTR(szDataFileName);
  hr = g_pSigDoc->verifySigDoc(fname, &lErr);
  ::SysFreeString(fname);
  if(FAILED(hr)) return -2;
  return (int)lErr;
}

int comVerifySignature(IComSignatureInfo *pSigInfo, char* szDataFileName)
{
  HRESULT hr = 0;
  long lErr = 0;
  BSTR fname = _com_util::ConvertStringToBSTR(szDataFileName);
  hr = g_pSigDoc->verifySignatureInfo(pSigInfo, fname, &lErr);
  ::SysFreeString(fname);
  if(FAILED(hr)) return -2;
  return (int)lErr;
}

int comCountDataFiles()
{
  HRESULT hr = 0;
  long l = 0;
  hr = g_pSigDoc->get_nDataFiles(&l);
  if(FAILED(hr)) return -2;
  return (int)l;
}

IComDataFile* comGetDataFile(int nIdx)
{
  IComDataFile* pDf = NULL;
  HRESULT hr = 0;
  long l = 0;

  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComDataFile"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comdatafile  %d\n", hr);
		return NULL;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComDataFile), (LPVOID *)&pDf);
  if(FAILED(hr)) {
        printf("Failed to connect to CComDataFile  %d\n", hr);
		return NULL;
  }
  hr = g_pSigDoc->getDataFile(nIdx, pDf);
  if(FAILED(hr)) {
        printf("Failed to get CComDataFile %d - %d\n", nIdx, hr);
		return NULL;
  }
  if(FAILED(hr)) return NULL;
  return pDf;
}

IComDataFile* comAddDataFile(char* szFileName, char* szMimeType, char* szContentType)
{
  IComDataFile* pDf = NULL;
  HRESULT hr = 0;
  long l = 0;
  VARIANT vDig;

  BSTR fname = _com_util::ConvertStringToBSTR(szFileName);
  BSTR mime = _com_util::ConvertStringToBSTR(szMimeType);
  BSTR content = _com_util::ConvertStringToBSTR(szContentType);
  BSTR dType = _com_util::ConvertStringToBSTR("");
  BSTR charset = _com_util::ConvertStringToBSTR("UTF-8");
  V_VT(&vDig)= VT_EMPTY;
  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComDataFile"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comdatafile  %d\n", hr);
		return NULL;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComDataFile), (LPVOID *)&pDf);
  if(FAILED(hr)) {
        printf("Failed to connect to CComDataFile  %d\n", hr);
		return NULL;
  }
  // convert fname to UTF-8
  //printf("Original filename: %s, len: %s\n", szFileName, strlen(szFileName));
  hr = g_pSigDoc->createDataFile(fname, content, mime, 0, vDig, 0, dType,charset,pDf);
  ::SysFreeString(fname);
  ::SysFreeString(content);
  ::SysFreeString(mime);
  ::SysFreeString(dType);
  ::SysFreeString(charset);
  if(FAILED(hr)) {
        printf("Failed to create CComDataFile %s / %s / %s - %d\n", szFileName, szMimeType, szContentType, hr);
		return NULL;
  }
  if(FAILED(hr)) return NULL;
  return pDf;
}

int comCalcDataFileSizeAndDigest(char* szId, char* szFileName)
{
  HRESULT hr = 0;
  long lErr = 0;
  BSTR fname = _com_util::ConvertStringToBSTR(szFileName);
  BSTR id = _com_util::ConvertStringToBSTR(szId);
  hr = g_pSigDoc->calculateDataFileSizeAndDigest(id, fname, 0, &lErr);
  ::SysFreeString(fname);
  ::SysFreeString(id);
  if(FAILED(hr)) {
        printf("Failed to calc CComDataFile %s / %s - %d\n", szId, szFileName, hr);
		return NULL;
  }
  return (int)lErr;
}

int comGetDataFileId(IComDataFile* pDf, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = pDf->get_szId(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetDataFileFileName(IComDataFile* pDf, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = pDf->get_szFileName(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetDataFileMimeType(IComDataFile* pDf, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = pDf->get_szMimeType(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetDataFileContentType(IComDataFile* pDf, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = pDf->get_szContentType(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetDataFileSize(IComDataFile* pDf, long *pL)
{
  HRESULT hr = 0;
  if(!pL) return -1;
  hr = pDf->get_nSize(pL);
  if(FAILED(hr)) return -2;
  return 0;
}


int comCountSignatures()
{
  HRESULT hr = 0;
  long l = 0;
  hr = g_pSigDoc->get_nSignatures(&l);
  if(FAILED(hr)) return -2;
  return (int)l;
}


IComSignatureInfo* comGetSignature(int nIdx)
{
  IComSignatureInfo* pSig = NULL;
  HRESULT hr = 0;
  long l = 0;

  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComSignatureInfo"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comsignature  %d\n", hr);
		return NULL;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComSignatureInfo), (LPVOID *)&pSig);
  if(FAILED(hr)) {
        printf("Failed to connect to CComSignatureInfo  %d\n", hr);
		return NULL;
  }
  hr = g_pSigDoc->getSignature(nIdx, pSig);
  if(FAILED(hr)) {
        printf("Failed to get CComSignatureInfo %d - %d\n", nIdx, hr);
		return NULL;
  }
  return pSig;
}

int comGetSignatureId(IComSignatureInfo* pSig, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = pSig->get_szId(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetSignatureTime(IComSignatureInfo* pSig, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf) return -1;
  memset(pBuf, 0, nLen);
  hr = pSig->get_szTimeStamp(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetSignatureCertHandle(IComSignatureInfo* pSig, long *pL)
{
  HRESULT hr = 0;
  if(!pL) return -1;
  hr = pSig->getSignCertData(pL);
  if(FAILED(hr)) return -2;
  return 0;
}

int comGetSignatureVerifiedByWrongHash(IComSignatureInfo* pSig, long *pL)
{
  HRESULT hr = 0;
  if(!pL) return -1;
  hr = pSig->get_verifiedByWrongDataFileHash(pL);
  if(FAILED(hr)) return -2;
  return 0;
}

int comGetCertNotAfter(long hCert, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  long lRet = 0;
  char *p1 = 0;
  if(!pBuf || !hCert) return -1;
  hr = g_pLib->getCertNotAfter(g_pSigDoc, hCert, &bstr, &lRet);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetCertNotBefore(long hCert, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  long lRet = 0;
  char *p1 = 0;
  if(!pBuf || !hCert) return -1;
  hr = g_pLib->getCertNotBefore(g_pSigDoc, hCert, &bstr, &lRet);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetCertSubject(long hCert, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  long lRet = 0;
  char *p1 = 0;
  if(!pBuf || !hCert) return -1;
  hr = g_pLib->getCertSubjectName(hCert, &bstr, &lRet);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetCertIssuer(long hCert, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  long lRet = 0;
  char *p1 = 0;
  if(!pBuf || !hCert) return -1;
  hr = g_pLib->getCertIssuerName(hCert, &bstr, &lRet);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetCertSerial(long hCert, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  long lRet = 0;
  char *p1 = 0;
  if(!pBuf || !hCert) return -1;
  hr = g_pLib->getCertSerialNumber(hCert, &bstr, &lRet);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}




int comCountNotaries()
{
  HRESULT hr = 0;
  long l = 0;
  hr = g_pSigDoc->get_nNotaries(&l);
  if(FAILED(hr)) return -2;
  return (int)l;
}

IComNotaryInfo* comGetNotary(int nIdx)
{
  IComNotaryInfo* pNot = NULL;
  HRESULT hr = 0;
  long l = 0;

  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComNotaryInfo"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comnotary  %d\n", hr);
		return NULL;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComNotaryInfo), (LPVOID *)&pNot);
  if(FAILED(hr)) {
        printf("Failed to connect to CComNotaryInfo  %d\n", hr);
		return NULL;
  }
  hr = g_pSigDoc->getNotaryInfo(nIdx, pNot);
  if(FAILED(hr)) {
        printf("Failed to get CComNotaryInfo %d - %d\n", nIdx, hr);
		return NULL;
  }
  return pNot;
}

IComNotaryInfo* comGetNotaryWithSigId(const char* szSigId)
{
  IComNotaryInfo* pNot = NULL;
  HRESULT hr = 0;
  long l = 0;
  BSTR bstr = _com_util::ConvertStringToBSTR(szSigId);
  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComNotaryInfo"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comnotary  %d\n", hr);
		return NULL;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComNotaryInfo), (LPVOID *)&pNot);
  if(FAILED(hr)) {
        printf("Failed to connect to CComNotaryInfo  %d\n", hr);
		return NULL;
  }
  hr = g_pSigDoc->getNotaryWithSigId(bstr, pNot);
  ::SysFreeString(bstr);
  if(FAILED(hr)) {
        printf("Failed to get CComNotaryInfo for sig %S - %d\n", szSigId, hr);
		return NULL;
  }
  return pNot;
}

int comGetNotaryId(IComNotaryInfo* pNot, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf || !pNot) return -1;
  hr = pNot->get_szId(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetNotaryResponderIdType(IComNotaryInfo* pNot, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf || !pNot) return -1;
  hr = pNot->get_szRespIdType(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetNotaryResponderIdName(IComNotaryInfo* pNot, char* pBuf, int* pLen)
{
  HRESULT hr = 0;
  VARIANT vt;
  if(!pBuf || !pNot || !pLen) return -1;
  int n = *pLen;
  memset(pBuf, 0, *pLen);
  hr = pNot->get_baRespId(&vt);
  if(FAILED(hr)) return -2; 
  SAFEARRAY* pArr = V_ARRAY(&vt);
  byte HUGEP* pData = 0;
  if(vt.vt == 8209) {
    *pLen = pArr->rgsabound->cElements;
    if (*pLen > 0 && *pLen < n) {
      SafeArrayAccessData(pArr, (void HUGEP**)&pData);
      memcpy(pBuf, pData, *pLen);
      SafeArrayUnaccessData(pArr);
    }
  }
  return 0;
}

int comGetNotaryTimeProduced(IComNotaryInfo* pNot, char* pBuf, int nLen)
{
  HRESULT hr = 0;
  BSTR bstr;
  char *p1 = 0;
  if(!pBuf || !pNot) return -1;
  hr = pNot->get_timeProduced(&bstr);
  if(FAILED(hr)) return -2;
  p1= _com_util::ConvertBSTRToString(bstr);
  ::SysFreeString(bstr);
  strncpy(pBuf, p1, nLen);
  delete [] p1;
  return 0;
}

int comGetNotaryCertHandle(IComNotaryInfo* pNot, long *pL)
{
  HRESULT hr = 0;
  if(!pL) return -1;
  hr = pNot->getNotCertData(pL);
  if(FAILED(hr)) return -2;
  return 0;
}

IComSignatureInfo* comSign(const char* pin, const char* manifest, 
			const char* city, const char* state, const char* zip, const char* country,
			const char* p12file, const char* p12pswd, const char* ocspUrl,
			const char* proxyHost, const char* proxyPort, int *pErr)
{
  IComSignatureInfo* pSig = NULL;
  HRESULT hr = 0;
  long l = 0;

  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComSignatureInfo"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comsignature  %d\n", hr);
		*pErr = -1;
		return NULL;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComSignatureInfo), (LPVOID *)&pSig);
  if(FAILED(hr)) {
        printf("Failed to connect to CComSignatureInfo  %d\n", hr);
		*pErr = -1;
		return NULL;
  }
  hr = g_pSigDoc->createSignatureInfo(pSig);
  if(FAILED(hr)) {
        printf("Failed to create CComSignatureInfo %d\n", hr);
		*pErr = -1;
		return NULL;
  }
  g_pSigDoc->addAllDocInfos(pSig);
  // add manifest
  if(manifest) {
	  VARIANT tfCertified, tfEncode;
	  BSTR bRole = _com_util::ConvertStringToBSTR(manifest);
	  V_BOOL(&tfCertified) = FALSE;
	  V_BOOL(&tfEncode) = FALSE;
	  *pErr = pSig->addSignerRole(tfCertified, bRole, tfEncode);
	  ::SysFreeString(bRole);
  }
  // add address
  if(city || state || zip || country) {
	  BSTR bCity = _com_util::ConvertStringToBSTR(city);
	  BSTR bState = _com_util::ConvertStringToBSTR(state);
	  BSTR bZip = _com_util::ConvertStringToBSTR(zip);
	  BSTR bCountry = _com_util::ConvertStringToBSTR(country);
	  *pErr = pSig->setSignatureProductionPlace(bCity, bState, bZip, bCountry);
	  ::SysFreeString(bCity);
	  ::SysFreeString(bState);
	  ::SysFreeString(bZip);
	  ::SysFreeString(bCountry);
  }

  // sign with CSP
  BSTR bPin = _com_util::ConvertStringToBSTR(pin);
  pSig->calculateSignatureWithCSPEstID(g_pSigDoc, 0, bPin, &l);
  ::SysFreeString(bPin);
  *pErr = (int)l;
  // get confirmation
  if(pSig && !(*pErr)) {
	  BSTR bPkcs12 = _com_util::ConvertStringToBSTR(p12file);
	  BSTR bP12Passwd = _com_util::ConvertStringToBSTR(p12pswd);
	  BSTR bUrl = _com_util::ConvertStringToBSTR(ocspUrl);
	  BSTR bProxyHost = _com_util::ConvertStringToBSTR(proxyHost);
	  BSTR bProxyPort = _com_util::ConvertStringToBSTR(proxyPort);
	  g_pSigDoc->getConfirmation(pSig, bPkcs12, bP12Passwd, bUrl, bProxyHost, bProxyPort, &l);
	  *pErr = (int)l;
      ::SysFreeString(bPkcs12);
	  ::SysFreeString(bP12Passwd);
	  ::SysFreeString(bUrl);
	  ::SysFreeString(bProxyHost);
	  ::SysFreeString(bProxyPort);
  }
  return pSig;
}

int comSearchStore()
{
  IComCertArray* pCertArr = NULL;
  IComCertSearch* pCertSearch = NULL;
  IComCertSearchStore *pCertSearchStore = NULL;
  IComCertificate* pCert = NULL;
  HRESULT hr = 0;
  VARIANT vt1, vt2;
  BSTR bstr;
  long l = 0, n = 0;
  char *p1 = 0;

  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComCertSearch"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comcertsearch  %d\n", hr);
		return -1;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComCertSearch), (LPVOID *)&pCertSearch);
  if(FAILED(hr)) {
        printf("Failed to connect to CComCertSearch  %d\n", hr);
		return -1;
  }
  hr = pCertSearch->put_nCertSearchBy(COM_CERT_SEARCH_BY_STORE);
  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComCertSearchStore"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comcertsearchstore  %d\n", hr);
		return -1;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComCertSearchStore), (LPVOID *)&pCertSearchStore);
  if(FAILED(hr)) {
        printf("Failed to connect to CComCertSearchStore  %d\n", hr);
		return -1;
  }
  hr = pCertSearch->getCertSearchStore(pCertSearchStore);
  if(FAILED(hr)) {
        printf("Failed to get adr of CComCertSearchStore  %d\n", hr);
		return -1;
  }
  hr = pCertSearchStore->addSearchType(COM_CERT_STORE_SEARCH_BY_ISSUER_DN);
  if(FAILED(hr)) {
        printf("Failed to add search type  %d\n", hr);
		return -2;
  }
  VariantInit(&vt1);
  SAFEARRAY* pArr = SafeArrayCreateVector(VT_BSTR, 0, 2);
  V_ARRAY(&vt1) = pArr;
  V_VT(&vt1) = (VT_ARRAY | VT_BSTR);
  BSTR HUGEP* pData = 0;
  SafeArrayAccessData(pArr, (void HUGEP**)&pData);
  USES_CONVERSION;
  pData[0] = SysAllocString(A2OLE("C=EE"));
  pData[1] = SysAllocString(A2OLE("O=AS Sertifitseerimiskeskus"));
  SafeArrayUnaccessData(pArr);
  hr = pCertSearchStore->put_saIssuerDNCriterias(vt1);
  if(FAILED(hr)) {
        printf("Failed to set issuer dn  %d\n", hr);
		return -2;
  }
  hr = pCertSearchStore->addSearchType(COM_CERT_STORE_SEARCH_BY_SUBJECT_DN);
  if(FAILED(hr)) {
        printf("Failed to add search type  %d\n", hr);
		return -2;
  }
  VariantInit(&vt2);
  SAFEARRAY* pArr2 = SafeArrayCreateVector(VT_BSTR, 0, 3);
  V_ARRAY(&vt2) = pArr2;
  V_VT(&vt2) = (VT_ARRAY | VT_BSTR);
  BSTR HUGEP* pData2 = 0;
  SafeArrayAccessData(pArr2, (void HUGEP**)&pData2);
  pData2[0] = SysAllocString(A2OLE("C=EE"));
  pData2[1] = SysAllocString(A2OLE("O=ESTEID"));
  pData2[2] = SysAllocString(A2OLE("OU=authentication"));
  SafeArrayUnaccessData(pArr2);
  hr = pCertSearchStore->put_saSubDNCriterias(vt2);
  if(FAILED(hr)) {
        printf("Failed to set subject dn  %d\n", hr);
		return -2;
  }  
  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComCertArray"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comcertarray  %d\n", hr);
		return -1;
  }
  hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComCertArray), (LPVOID *)&pCertArr);
  if(FAILED(hr)) {
        printf("Failed to connect to CComCertArray  %d\n", hr);
		return -1;
  }
  l = 0;
  hr = pCertArr->searchBySearchStore(pCertSearchStore, &l);
  if(FAILED(hr) || l) {
        printf("Failed to search certs  %d err: %d\n", hr, l);
		return -3;
  }
  hr = pCertArr->get_nCerts(&n);
  if(FAILED(hr)) {
        printf("Failed to get number of found certs  %d\n", hr);
		return -3;
  }
  printf("Found  %d certs\n", n);
  hr = CLSIDFromProgID(OLESTR("DigiDocLibCOM.ComCertificate"), &g_clsid);
  if(FAILED(hr)) {
        printf("Failed to find clsid of comcertificate  %d\n", hr);
		return -1;
  }
  for(int j = 0; j < n; j++) {
    hr = CoCreateInstance(g_clsid, NULL, CLSCTX_INPROC_SERVER, __uuidof(IComCertificate), (LPVOID *)&pCert);
    if(FAILED(hr)) {
        printf("Failed to connect to CComCertificate  %d\n", hr);
		return -1;
    }
    hr = pCertArr->getCertByIndex(j, pCert);
    if(FAILED(hr)) {
        printf("Failed to read cert %d - %d\n", j, hr);
		return -3;
    }
    hr = pCert->get_szPropSubjectName(&bstr);
    p1= _com_util::ConvertBSTRToString(bstr);
    ::SysFreeString(bstr);
    printf("Cert: %d - %s\n", j, p1);
    delete [] p1;
    p1 = 0;
    pCert->Release();
  }
  pCertArr->Release();
  pCertSearchStore->Release();
  pCertSearch->Release();
  return 0;
}


//==========< command handlers >====================

//--------------------------------------------------
// Creates a new signed doc
//--------------------------------------------------
int cmdNew(char* pFormat, char* pVersion)
{
  int err = 0;
  char* format = pFormat, *version = pVersion;

  if(!p_szOutFile) {
    err = -2;
    printf("No output file specified");
    return err;
  }
  if(!format || !strlen(format))
    format = "DIGIDOC-XML";
  if(!version || !strlen(version))
    version = "1.3";
  if(format && version) {
	  //printf("new: %s %s\n", format, version);
     err = comNewSignedDoc(format, version);
	 g_bDdocExists = true;
  } else {
    err = -3;
    printf("Error finding new document format or version");
  }
  return err;
}



//--------------------------------------------------
// Adds a DataFile to signed doc
//--------------------------------------------------
int cmdAddDataFile( char* file, 
		    char* mime,  char* content)
{
  int err = 0;
  IComDataFile*  pDf = NULL;
  char *p1 = 0, *p2 = 0, buf1[300];

  // if there was no new command then implicitly create a new document
  if(!g_bDdocExists) {
    err = cmdNew(NULL, NULL);
    if(err) return err; 
  }
  // add a file	
  ddocConvertInput((const char*)file, &p1);
  ddocConvertInput((const char*)mime, &p2);
  pDf = comAddDataFile(p1, p2, content);
  //freeLibMem(p1);
  if(pDf) {
	err = comGetDataFileId(pDf, buf1, sizeof(buf1));
	if(!err)
    err = comCalcDataFileSizeAndDigest(buf1, p1);
    printf("DF calc: %d\n", err);
    pDf->Release();
  }
  return err;
}

char* getArg(int argc, _TCHAR* argv[], int nArg, boolean bCmd)
{
	char *p = NULL;
	if(nArg >= 0 && nArg < argc) {
		CW2A a(argv[nArg]);
		p = strdup(a);
		if(p && ((!bCmd && p[0] == '-') || (bCmd && p[0] != '-'))) {
			free(p);
			p = NULL;
		}
	}
	return p;
}

//--------------------------------------------------
// Create digidoc and adds datafiles
//--------------------------------------------------
int runNewCmds(int argc, _TCHAR* argv[])
{
  int err = 0, i;
  char *p1 = NULL, *p2 = NULL, *p3 = NULL;

  for(i = 1; (err == 0) && (i < argc); i++) {
	p1 = getArg(argc, argv, i, true);
	if(p1 && !strcmp(p1, "-new")) {
		p2 = getArg(argc, argv, i+1, false);
		if(p2)
		p3 = getArg(argc, argv, i+2, false);
		err = cmdNew(p2, p3);
		free(p2);
		free(p3);
		if(!err) return err;
    }
	free(p1);
  }
  return err;
}

//--------------------------------------------------
// Create digidoc and adds datafiles
//--------------------------------------------------
int runAddCmds(int argc, _TCHAR* argv[])
{
  int err = 0, i;
  char *p1 = NULL, *p2 = NULL, *p3 = NULL;

  for(i = 1; (err == 0) && (i < argc); i++) {
	p1 = getArg(argc, argv, i, true);
	if(p1 && !strcmp(p1, "-add") && argc > i+2) { // all commands and options must start with -
		p2 = getArg(argc, argv, i+1, false);
		if(p2)
		p3 = getArg(argc, argv, i+2, false);
		//printf("add: %s %s\n", p2, p3);
		err = cmdAddDataFile(p2, p3, "EMBEDDED_BASE64");
		free(p2);
		free(p3);
	  i += 2;
    }
	free(p1);
  }
  return err;
}


void printCertData(long hCert)
{
  int  err = 0;
  char buf1[300];

  err = comGetCertSerial(hCert, buf1, sizeof(buf1));
  printf("X509Certificate|%s", buf1);
  err = comGetCertIssuer(hCert, buf1, sizeof(buf1));
  printf("|%s", buf1);
  err = comGetCertSubject(hCert, buf1, sizeof(buf1));
  printf("|%s", buf1);
  err = comGetCertNotBefore(hCert, buf1, sizeof(buf1));
  printf("|%s", buf1);
  err = comGetCertNotAfter(hCert, buf1, sizeof(buf1));
  printf("|%s\n", buf1);
}

//--------------------------------------------------
// Verifys sigantures and notaries
//--------------------------------------------------
int cmdVerify()
{
  int err = 0, d, l, n, err2 = 0;
  char buf1[300], id[100];
  long l1, lCert1 = 0, lCert2 = 0, l2=0;

  //printf("verify\n");
  printf("SignedDoc|");
  comGetSigDocFormat(buf1, sizeof(buf1));
  printf("%s|", buf1);
  comGetSigDocVer(buf1, sizeof(buf1));
  printf("%s|", buf1);
  comCleanErrors();
  err2 = comVerifySignedDoc(NULL);
  printf("%d\n", err2);
  //comGetErrors(buf1, sizeof(buf1));
  //printf("%s\n", buf1);
  d = comCountDataFiles();
  for(l = 0; l < d; l++) {
	IComDataFile* pDf = comGetDataFile(l);
	comGetDataFileId(pDf, buf1, sizeof(buf1));
	printf("DataFile|%s", buf1);
    comGetDataFileFileName(pDf, buf1, sizeof(buf1));
	comGetDataFileSize(pDf, &l1);
	printf("|%s|%d", buf1, l1);
    comGetDataFileMimeType(pDf, buf1, sizeof(buf1));
	printf("|%s", buf1);
    comGetDataFileContentType(pDf, buf1, sizeof(buf1));
	printf("|%s\n", buf1);
	pDf->Release();
  }
  d = comCountSignatures();
  for(l = 0; l < d; l++) {
	IComSignatureInfo* pSig = comGetSignature(l);
	comGetSignatureId(pSig, buf1, sizeof(buf1));
	strncpy(id, buf1, sizeof(id));
	printf("Signature|%s", buf1);
	err = comGetSignatureCertHandle(pSig, &lCert1);
	if(lCert1) {
	comGetCertSubject(lCert1, buf1, sizeof(buf1));
	printf("|%s", buf1);
	}
	comCleanErrors();
        n = comVerifySignature(pSig, NULL);
	comGetErrors(buf1, sizeof(buf1));
	printf("|%d|%s\n", n, buf1);
	l2 = 0;
	comGetSignatureVerifiedByWrongHash(pSig, &l2);
	if(l2)
	  printf("WARNING Signature has been verified by wrong DataFile hash");
	if(lCert1)
	printCertData(lCert1);
	
	// show notary
	IComNotaryInfo* pNot = comGetNotaryWithSigId(id);
	comGetNotaryId(pNot, buf1, sizeof(buf1));
	strncpy(id, buf1, sizeof(id));
	printf("RevocationValues|%s", buf1);
	n = sizeof(buf1);
	comGetNotaryResponderIdName(pNot, buf1, &n);
	printf("|%s", buf1);
	comGetNotaryTimeProduced(pNot, buf1, sizeof(buf1));
	printf("|%s\n", buf1);
	comGetNotaryCertHandle(pNot, &lCert2);
	if(lCert2)
	printCertData(lCert2);
    if(pNot)
	pNot->Release();
	pSig->Release();
  }
  // must run clean & verify again because we had to clean errors before each separate signature
  comCleanErrors();
  err2 = comVerifySignedDoc(NULL);
  return err2;
}


//--------------------------------------------------
// Create digidoc and adds datafiles
//--------------------------------------------------
int runExtractCmds(int argc, _TCHAR* argv[])
{
  int err = 0, i;
  char *p1 = NULL, *p2 = NULL, *p3 = NULL, *p2u = 0, *p3u = 0;

  for(i = 1; (err == 0) && (i < argc); i++) {
	p1 = getArg(argc, argv, i, true);
    if(p1 && !strcmp(p1, "-extract") && (argc > i+2)) {
	  p2 = getArg(argc, argv, i+1, false);
	  if(p2) {
		ddocConvertInput((const char*)p2, &p2u);
	    p3 = getArg(argc, argv, i+2, false);
	  }
	  if(p3) 
		  ddocConvertInput((const char*)p3, &p3u);
	  err = comExtractDataFile(p_szInFile, p2u, p3u);
	  printf("\nDataFile|%s|%s|%d", p2, p3, err);
	  free(p2);
	  free(p3);
	  i += 2;	
    }
	free(p1);
  }
  return err;
}


//--------------------------------------------------
// Verfys sigantures and notaries
//--------------------------------------------------
int runVerifyCmds(int argc, _TCHAR* argv[])
{
  int err = 0, i;
  
  for(i = 1; !err && (i < argc); i++) {
    if(argv[i][0] == '-') { // all commands and options must start with -
	  CW2A pszA(argv[i]);
      // create new digidoc
	  if(!strcmp(pszA, "-verify")) {
		err = cmdVerify();
      }
    }
  }
  return err;	
}


//--------------------------------------------------
// Handles signature commands
//--------------------------------------------------
int runSignCmds(int argc, _TCHAR* argv[])
{
  int err = 0, i;
  IComSignatureInfo* pSig = NULL;
  char *p1 = NULL, *p2 = NULL, *p3 = NULL, *p4 = NULL, *p5 = NULL, *p6 = NULL, *p7 = NULL;
  char *p3u = NULL, *p4u = NULL, *p5u = NULL, *p6u = NULL, *p7u = NULL;

  for(i = 1; !err && (i < argc); i++) {
	p1 = getArg(argc, argv, i, true);
    if(p1 && p1[0] == '-') { // all commands and options must start with -
      // create new digidoc
      if(!strcmp(p1, "-sign")) {
	  p2 = getArg(argc, argv, i+1, false);
	  if(p2)
	  p3 = getArg(argc, argv, i+2, false);
	  if(p3) {
		ddocConvertInput((const char*)p3, &p3u);
		p4 = getArg(argc, argv, i+3, false);
	  }
	  if(p4) {
		ddocConvertInput((const char*)p4, &p4u);
		p5 = getArg(argc, argv, i+4, false);
	  }
	  if(p5) {
		ddocConvertInput((const char*)p5, &p5u);
		p6 = getArg(argc, argv, i+5, false);
	  }
	  if(p6) {
		ddocConvertInput((const char*)p6, &p6u);
		p7 = getArg(argc, argv, i+6, false);
	  }
	  if(p7)
		ddocConvertInput((const char*)p7, &p7u);
	  pSig = comSign((const char*)p2,
			(const char*)p3u, (const char*)p4u, (const char*)p5u, (const char*)p6u, (const char*)p7u,
			NULL, NULL, NULL, NULL, NULL, &err);
	  free(p2);
	  free(p3);
	  free(p4);
	  free(p5);
	  free(p6);
	  free(p7);
	  if(pSig)
		  pSig->Release();
      }
    }
	free(p1);
  }
  return err;
}

//--------------------------------------------------
// Handles list commands
//--------------------------------------------------
int runListCmds(int argc, _TCHAR* argv[])
{
  int err = 0, i, nFound = 0;

  for(i = 1; !err && (i < argc); i++) {
	CW2A pszA(argv[i]);
    if(pszA[0] == '-') { // all commands and options must start with -
      // create new digidoc
      if(!strcmp(pszA, "-list")) {
		nFound = 1;
      }
    }
  }
  if(nFound) {
    err = cmdVerify();
  }
  return err;
}

//--------------------------------------------------
// Handles list commands
//--------------------------------------------------
int runListCertsCmds(int argc, _TCHAR* argv[])
{
  int err = 0, i, nFound = 0;

  for(i = 1; !err && (i < argc); i++) {
	CW2A pszA(argv[i]);
    if(pszA[0] == '-') { // all commands and options must start with -
      // create new digidoc
      if(!strcmp(pszA, "-list-certs")) {
		nFound = 1;
      }
    }
  }
  if(nFound) {
    err = comSearchStore();
  }
  return err;
}


int _tmain(int argc, _TCHAR* argv[])
{
    time_t t1 = 0, t2 = 0;
	int err = 0, l1 = 0;
	char buf1[3000];
	
	// check program arguments
	if(argc <= 1) {
      printUsage();
      exit(0);
    }
	err = checkProgArguments(argc, argv);
	// connect to com
	err = comConnect();
	printf("%s", g_szProgNameVer);
	err = comGetLibName(buf1, sizeof(buf1));
	printf("|LIB|%s", buf1);
	err = comGetLibVer(buf1, sizeof(buf1));
	printf("|%s", buf1);
	err = comGetComVer(buf1, sizeof(buf1));
	printf("|COM| %s\n", buf1);
	comSetClientNameVer(g_szProgNameVer);
	//comSetDebugFilePath("z:\\digidoccom.log");
	//scomSetDebugLevel(3);
    time(&t1);
	// read input file if necessary
    if(p_szInFile && !err) {
      err = comReadSignedDoc(p_szInFile);
	  g_bDdocExists = true;
	}
	// new digidocs
    if(!err)
      err = runNewCmds(argc, argv);
	// add data files
    if(!err)
      err = runAddCmds(argc, argv);
	// run signature commands
    if(!err)
      err = runSignCmds(argc, argv);
	// verify signatures
    if(!err)
      err = runVerifyCmds(argc, argv);
    // extract datafiles
    if(!err)
      err = runExtractCmds(argc, argv);
	// write output file
    if(p_szOutFile && !err) {
		//printf("write: %s to %s\n", p_szInFile, p_szOutFile);
      err = comWriteSignedDoc(p_szInFile, p_szOutFile);
	  //printf("Wrote err: %d\n", err);
	}
    // list digidoc or encrypted documents
    if(!err)
      err = runListCmds(argc, argv);
    if(!err)
      err = runListCertsCmds(argc, argv);
	  //printf("End work err: %d\n", err);
    time(&t2);
	// list all errors
	if(err) {
		//printf("Get errs\n");
	  comGetErrors(buf1, sizeof(buf1));
	  printf("ERRORS: %s\n", buf1);
	}
	printf("\n%s - time: %ld sec result: %s\n", g_szProgNameVer, (long)(t2-t1), (err ? "failure" : "success"));
  
	//cmdLibNameAndVer(pLib);

	 
	// cleanup
	comDisconnect();
    return 0;
}

