// ComCertSearch.cpp : Implementation of CComCertSearch
#include "stdafx.h"
#include "DigiDocLibCOM_h.h"
#include "ComCertSearch.h"
#include "Converters.h"

/////////////////////////////////////////////////////////////////////////////
// CComCertSearch


//////////////////////////////////////////////////////////////
//        properties
//////////////////////////////////////////////////////////////

STDMETHODIMP CComCertSearch::get_nCertSearchBy(long *pVal)
{
	if(m_pCertSearch)
		*pVal = m_pCertSearch->searchType;
	return S_OK;
}

STDMETHODIMP CComCertSearch::put_nCertSearchBy(long newVal)
{
	if(m_pCertSearch)
		m_pCertSearch->searchType = newVal;
	return S_OK;
}

STDMETHODIMP CComCertSearch::get_szX509FileName(BSTR *pVal)
{
	if(m_pCertSearch && m_pCertSearch->x509FileName)
		Convert::copysz2BSTR_withAlloc(m_pCertSearch->x509FileName, pVal);
	return S_OK;
}

STDMETHODIMP CComCertSearch::put_szX509FileName(BSTR newVal)
{
	if(m_pCertSearch)
		CertSearch_setX509FileName(m_pCertSearch, GetBSTR(newVal));
	return S_OK;
}


STDMETHODIMP CComCertSearch::get_szKeyFileName(BSTR *pVal)
{
	if(m_pCertSearch && m_pCertSearch->keyFileName)
		Convert::copysz2BSTR_withAlloc(m_pCertSearch->keyFileName, pVal);
	return S_OK;
}

STDMETHODIMP CComCertSearch::put_szKeyFileName(BSTR newVal)
{
	if(m_pCertSearch)
		CertSearch_setKeyFileName(m_pCertSearch, GetBSTR(newVal));
	return S_OK;
}

STDMETHODIMP CComCertSearch::get_szPkcs12FileName(BSTR *pVal)
{
	if(m_pCertSearch && m_pCertSearch->pkcs12FileName)
		Convert::copysz2BSTR_withAlloc(m_pCertSearch->pkcs12FileName, pVal);
	return S_OK;
}

STDMETHODIMP CComCertSearch::put_szPkcs12FileName(BSTR newVal)
{
	if(m_pCertSearch)
		CertSearch_setPkcs12FileName(m_pCertSearch, GetBSTR(newVal));
	return S_OK;
}

STDMETHODIMP CComCertSearch::get_pswd(BSTR *pVal)
{
	if(m_pCertSearch && m_pCertSearch->pswd)
		Convert::copysz2BSTR_withAlloc(m_pCertSearch->pswd, pVal);
	return S_OK;
}

STDMETHODIMP CComCertSearch::put_pswd(BSTR newVal)
{
	if(m_pCertSearch)
		CertSearch_setPasswd(m_pCertSearch, GetBSTR(newVal));
	return S_OK;
}


//////////////////////////////////////////////////////////////
//        methods
//////////////////////////////////////////////////////////////

STDMETHODIMP CComCertSearch::getCertSearchStore(IComCertSearchStore* pStoreTo)
{
  if (m_pCertSearch->certSearchStore == (CertSearchStore*)0)
    m_pCertSearch->certSearchStore = ::CertSearchStore_new();
  ((CComCertSearchStore*)pStoreTo)->shiftStructPointer(
    m_pCertSearch->certSearchStore);

	return S_OK;
}

STDMETHODIMP CComCertSearch::lookup(VARIANT* pRetBool)
{

	VariantInit(pRetBool);
	V_VT(pRetBool) = VT_BOOL;
	if(m_pCertSearch) {
		X509* pCert = findCertificate(m_pCertSearch);
		V_BOOL(pRetBool) = ((pCert == NULL) ? FALSE : TRUE);
		if (pCert != NULL)
			::X509_free(pCert); // TODO: delete!!!
	}
	int toDbg = V_BOOL(pRetBool) ? 1 : 0;
	return S_OK;
}


//////////////////////////////////////////////////////////////
//        helpers
//////////////////////////////////////////////////////////////
CComCertSearch::CComCertSearch() {
  m_pointerMode = false;
  m_pCertSearch = ::CertSearch_new();
}

CComCertSearch::~CComCertSearch() {
  if (!m_pointerMode)
    freeStruct();
}

CertSearch* CComCertSearch::getCertSearch() {
  return m_pCertSearch;
}

void CComCertSearch::shiftStructPointer(CertSearch* pNew) {
  if (!m_pointerMode)
    freeStruct();
  m_pCertSearch = pNew;
  m_pointerMode = true;
}

// frees memory allocated for struct
void CComCertSearch::freeStruct() {
	if(!m_pointerMode && m_pCertSearch) {
		::CertSearch_free(m_pCertSearch);
		m_pCertSearch = NULL;
	}
}

void CComCertSearch::reset() {
	m_pCertSearch->searchType = 0;
	if (m_pCertSearch->x509FileName != (char*)0)
    delete m_pCertSearch->x509FileName;
	if (m_pCertSearch->keyFileName != (char*)0)
    delete m_pCertSearch->keyFileName;
	if (m_pCertSearch->pkcs12FileName != (char*)0)
    delete m_pCertSearch->pkcs12FileName;
	if (m_pCertSearch->pswd != (char*)0)
    delete m_pCertSearch->pswd;
	if (m_pCertSearch->certSearchStore != (CertSearchStore*)0)
    delete m_pCertSearch->certSearchStore;
}


//////////////////////////////////////////////////////////////
//        helper class
//////////////////////////////////////////////////////////////

CertHolderInScope::CertHolderInScope(CComCertSearch& lookup) {
  cert = ::findCertificate(lookup.getCertSearch());
}

CertHolderInScope::CertHolderInScope(X509* rememberCert) {
  cert = rememberCert;
}

CertHolderInScope::~CertHolderInScope() {
  if (cert)
		::X509_free(cert);
}

