/*

Wrap of DigiCoc library CertSearch structure to provide COM-interface
Copyright (C) AS Sertifitseerimiskeskus

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

GNU Lesser General Public Licence is available at
http://www.gnu.org/copyleft/lesser.html

*/
// ComCertSearch.h : Declaration of the CComCertSearch

#ifndef __COMCERTSEARCH_H_
#define __COMCERTSEARCH_H_

#include "resource.h"       // main symbols

#include "Converters.h"
#include "ComCertSearchStore.h"

/////////////////////////////////////////////////////////////////////////////
// CComCertSearch
class ATL_NO_VTABLE CComCertSearch : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CComCertSearch, &CLSID_ComCertSearch>,
	public IDispatchImpl<IComCertSearch, &IID_IComCertSearch, &LIBID_DIGIDOCLIBCOMLib>
{
public:
	CComCertSearch();
	~CComCertSearch();

DECLARE_REGISTRY_RESOURCEID(IDR_COMCERTSEARCH)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CComCertSearch)
	COM_INTERFACE_ENTRY(IComCertSearch)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

// IComCertSearch
public:
	STDMETHOD(lookup)(/*[out, retval]*/ VARIANT* pRetBool);
	STDMETHOD(getCertSearchStore)(/*[in]*/ IComCertSearchStore* pStoreTo);
	STDMETHOD(get_pswd)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_pswd)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_szPkcs12FileName)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_szPkcs12FileName)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_szKeyFileName)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_szKeyFileName)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_szX509FileName)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_szX509FileName)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_nCertSearchBy)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_nCertSearchBy)(/*[in]*/ long newVal);

  CertSearch* getCertSearch();
  void shiftStructPointer(CertSearch* pNew);

  // frees memory allocated for struct
  void freeStruct();

protected:
  CertSearch* m_pCertSearch;
  bool m_pointerMode;

  void reset();
};



//////////////////////////////////////////////////////////////
//        helper class
//////////////////////////////////////////////////////////////
class CertHolderInScope {
public:
  X509* cert;

  CertHolderInScope(CComCertSearch& lookup);
  CertHolderInScope(X509* rememberCert);
  ~CertHolderInScope();

  bool isAvailable() { return (cert ? true : false); }
};

#endif //__COMCERTSEARCH_H_
