// ComDataFile.cpp : Implementation of CComDataFile
#include "stdafx.h"
#include "DigiDocLibCOM_h.h"
#include "ComDataFile.h"

#include "Converters.h"
#include "libdigidoc/DigiDocConvert.h"
#include "libdigidoc/DigiDocMem.h"
//#include "DigiDocLib.h"

/////////////////////////////////////////////////////////////////////////////
// CComDataFile

//////////////////////////////////////////////////////////////
//        properties
//////////////////////////////////////////////////////////////

STDMETHODIMP CComDataFile::get_szId(BSTR *pVal)
{
	if(m_pDataFile)
		Convert::out_retval_sz2BSTR(m_pDataFile->szId, pVal);
	return S_OK;
}

STDMETHODIMP CComDataFile::get_szFileName(BSTR *pVal)
{
	DigiDocMemBuf mbuf1;
	int err = 0;

	mbuf1.pMem = 0;
	mbuf1.nLen = 0;
	if(m_pDataFile && m_pDataFile->szFileName) {
		ddocMemSetLength(&mbuf1, strlen(m_pDataFile->szFileName)+1);
		err = getDataFileFileName(m_pSignedDoc, m_pDataFile, 
			(char*)mbuf1.pMem, mbuf1.nLen);
	}
    Convert::out_retval_sz2BSTR((char*)mbuf1.pMem, pVal);
	ddocMemBuf_free(&mbuf1);
	return S_OK;
}

STDMETHODIMP CComDataFile::get_szMimeType(BSTR *pVal)
{
	if(m_pDataFile)
		Convert::out_retval_sz2BSTR(m_pDataFile->szMimeType, pVal);
	return S_OK;
}

STDMETHODIMP CComDataFile::get_szContentType(BSTR *pVal)
{
	if(m_pDataFile)
		Convert::out_retval_sz2BSTR(m_pDataFile->szContentType, pVal);
	return S_OK;
}

STDMETHODIMP CComDataFile::get_nSize(long *pVal)
{
	if(m_pDataFile)
		*pVal = m_pDataFile->nSize;
	return S_OK;
}

STDMETHODIMP CComDataFile::get_szDigestType(BSTR *pVal)
{
	if(m_pDataFile)
		Convert::out_retval_sz2BSTR(m_pDataFile->szDigestType, pVal);
	return S_OK;
}

STDMETHODIMP CComDataFile::get_baDigest(VARIANT *pVal)
{
	DigiDocMemBuf *pMBuf;

	if(m_pDataFile) {
		pMBuf = ddocDataFile_GetDigestValue(m_pDataFile);
		Convert::out_retval_ac2VARIANT((const char*)pMBuf->pMem, pMBuf->nLen, pVal);
	}
	return S_OK;
}

STDMETHODIMP CComDataFile::get_nDigestLen(long *pVal)
{
	DigiDocMemBuf *pMBuf;

	if(m_pDataFile) {
		pMBuf = ddocDataFile_GetDigestValue(m_pDataFile);
		*pVal = pMBuf->nLen;
	}
	return S_OK;
}

STDMETHODIMP CComDataFile::get_nAttributes(long *pVal)
{
	if(m_pDataFile)
		*pVal = m_pDataFile->nAttributes;
	return S_OK;
}

STDMETHODIMP CComDataFile::get_szaAttNames(VARIANT *pVal)
{
	if(!m_pDataFile || (m_pDataFile && m_pDataFile->pAttNames == (char**)0)) {
		V_VT(pVal)= VT_EMPTY;
	} else {
		SAFEARRAY* pArr = SafeArrayCreateVector(VT_BSTR, 0, m_pDataFile->nAttributes);

		V_ARRAY(pVal) = pArr;
		V_VT(pVal) = (VT_ARRAY | VT_BSTR);

		BSTR HUGEP* pData;
		SafeArrayAccessData(pArr, (void HUGEP**)&pData);

		USES_CONVERSION;
		for(int i = 0; i < m_pDataFile->nAttributes; i++) {
			SetBSTR(&pData[i], m_pDataFile->pAttNames[i]); //pData[i] = SysAllocString(A2OLE(getDataFile()->pAttNames[i])); // TODO it's slow, use func call
		}
		SafeArrayUnaccessData(pArr);
	}
	return S_OK;
}

STDMETHODIMP CComDataFile::get_szaAttValues(VARIANT *pVal)
{
	if (!m_pDataFile || (m_pDataFile && m_pDataFile->pAttValues == (char**)0)) {
		V_VT(pVal)= VT_EMPTY;
	} else {
		SAFEARRAY* pArr = SafeArrayCreateVector(VT_BSTR, 0, m_pDataFile->nAttributes);

		V_ARRAY(pVal) = pArr;
		V_VT(pVal) = (VT_ARRAY | VT_BSTR);

		BSTR HUGEP* pData;
		SafeArrayAccessData(pArr, (void HUGEP**)&pData);

		USES_CONVERSION;
		for(int i = 0; i < m_pDataFile->nAttributes; i++) {
			SetBSTR(&pData[i], m_pDataFile->pAttValues[i]); //pData[i] = SysAllocString(A2OLE(getDataFile()->pAttValues[i])); // TODO it's slow, use func call
		}
		SafeArrayUnaccessData(pArr);
	}
	return S_OK;
}


//////////////////////////////////////////////////////////////
//        methods
//////////////////////////////////////////////////////////////

STDMETHODIMP CComDataFile::addAttribute(BSTR szAttrName, BSTR szAttrValue)
{
	USES_CONVERSION;

	if(m_pDataFile)
		::addDataFileAttribute(m_pDataFile, GetBSTR(szAttrName), GetBSTR(szAttrValue));
	return S_OK;
}

STDMETHODIMP CComDataFile::getAttributeName(long nIndex, BSTR *szRetName)
{
  char* pName = NULL;
  char* pValue = NULL;
  USES_CONVERSION;

  if(m_pDataFile) {
	::getDataFileAttribute(m_pDataFile, nIndex, &pName, &pValue);
	if(pName)
		SetBSTR(szRetName, pName); //*szRetName = SysAllocString(A2OLE(pName));
  }
  return S_OK;
}

STDMETHODIMP CComDataFile::getAttributeValue(long nIndex, BSTR *szRetValue)
{
  char* pName = NULL;
  char* pValue = NULL;
  USES_CONVERSION;

  if(m_pDataFile) {
	::getDataFileAttribute(m_pDataFile, nIndex, &pName, &pValue);
	if(pValue)
		SetBSTR(szRetValue, pValue); //*szRetValue = SysAllocString(A2OLE(pValue));
  }
  return S_OK;
}



////////////////////////////////////////////////////////////////////
//    helpers
////////////////////////////////////////////////////////////////////
CComDataFile::CComDataFile() {
  m_pDataFile = NULL;
  m_pSignedDoc = NULL;
}

CComDataFile::~CComDataFile() {
	/*if(m_pDataFile) {
		::DataFile_free(m_pDataFile);
		m_pDataFile = NULL;
	}*/
}


void CComDataFile::shiftStructPointer(DataFile* pNew) {
  /*if(m_pDataFile) {
	::DataFile_free(m_pDataFile);
	m_pDataFile = NULL;
  }*/
  m_pDataFile = pNew;
}



