// ComDigiDocLib.cpp : Implementation of CComDigiDocLib
#include "stdafx.h"
#include "libdigidoc/DigiDocConvert.h"
#include "libdigidoc/DigiDocGlobals.h"
#include "libdigidoc/DigiDocDfExtract.h"
#include "libdigidoc/DigiDocDebug.h"
#include "DigiDocLibCOM_h.h"
#include "ComDigiDocLib.h"
#include "ComCertificate.h"
#include <comdef.h>
#include "config.h"


/////////////////////////////////////////////////////////////////////////////
// CComDigiDocLib

CComDigiDocLib :: CComDigiDocLib()
	: m_pProfile(0)
{
	m_pProfile = TSAProfile_init();
}

CComDigiDocLib :: ~CComDigiDocLib()
{
	TSAProfile_free();
}

STDMETHODIMP CComDigiDocLib::getLibName(BSTR *ret)
{
  USES_CONVERSION;
  SetBSTR(ret, (char *)::getLibName()); //*ret = SysAllocString(A2OLE(::getLibName()));

  return S_OK;
}

STDMETHODIMP CComDigiDocLib::getLibVersion(BSTR *ret)
{
  USES_CONVERSION;

  //KERT
  SetBSTR(ret,DIGIDOC_VERSION);
  //SetBSTR(ret, (char *)::getLibVersion()); //*ret = SysAllocString(A2OLE(::getLibVersion()));
  return S_OK;
}


STDMETHODIMP CComDigiDocLib::getComVersion(BSTR *ret){
	// TODO: Add your implementation code here
	SetBSTR(ret,DIGIDOC_VERSION);
	return S_OK;
}




// http://www.devguy.com/fp/Tips/COM/bstr.htm

STDMETHODIMP CComDigiDocLib::get_TsaUrl(BSTR *pVal)
{
	USES_CONVERSION;

	if(m_pProfile)
	  Convert::copysz2BSTR_withAlloc(m_pProfile->g_szTsaUrl, pVal);

	return S_OK;
}

STDMETHODIMP CComDigiDocLib::put_TsaUrl(BSTR newVal)
{
	USES_CONVERSION;

	if(m_pProfile)
	  strncpy(m_pProfile->g_szTsaUrl, OLE2T(newVal), sizeof(m_pProfile->g_szTsaUrl));

	return S_OK;
}


STDMETHODIMP CComDigiDocLib::get_DebugFilePath(BSTR *pVal)
{
	USES_CONVERSION;

	if(m_pProfile)
		Convert::copysz2BSTR_withAlloc(m_pProfile->m_szDebugFilePath, pVal);


    return S_OK;
}



STDMETHODIMP CComDigiDocLib::put_DebugFilePath(BSTR newVal)
{
	USES_CONVERSION;

	if(m_pProfile)
	{
	  strncpy(m_pProfile->m_szDebugFilePath, OLE2T(newVal), sizeof(m_pProfile->m_szDebugFilePath));
  	  createOrReplacePrivateConfigItem(NULL, "DEBUG_FILE", m_pProfile->m_szDebugFilePath);
	}
    return S_OK;
}


STDMETHODIMP CComDigiDocLib::get_DebugLevel(LONG *pVal)
{
	if(m_pProfile)
		*pVal = m_pProfile->m_nDebugLevel;
    return S_OK;
}



STDMETHODIMP CComDigiDocLib::put_DebugLevel(LONG newVal)
{
	if(m_pProfile) {
		m_pProfile->m_nDebugLevel = newVal;
		char _stack_buffer[30];
		sprintf(_stack_buffer, "%ld", newVal);
		createOrReplacePrivateConfigItem(NULL, "DEBUG_LEVEL", _stack_buffer);
	}
    return S_OK;
}

STDMETHODIMP CComDigiDocLib::get_AddTimeStamp(VARIANT_BOOL *pVal)
{
	//USES_CONVERSION;
	if(m_pProfile)
		*pVal = (VARIANT_BOOL)m_pProfile->g_bAddSignatureTimeStamp;
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::put_AddTimeStamp(VARIANT_BOOL newVal)
{
	//USES_CONVERSION;

	if(m_pProfile)
		m_pProfile->g_bAddSignatureTimeStamp = (int) newVal;
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::get_AddSigAndRefsTimeStamp(VARIANT_BOOL *pVal)
{
	if(m_pProfile)
		*pVal = (VARIANT_BOOL)m_pProfile->g_bAddSigAndRefsTimeStamp;
    return S_OK;
}



STDMETHODIMP CComDigiDocLib::put_AddSigAndRefsTimeStamp(VARIANT_BOOL newVal)
{
	if(m_pProfile)
		m_pProfile->g_bAddSigAndRefsTimeStamp = (int) newVal;
    return S_OK;
}

STDMETHODIMP CComDigiDocLib::get_TimeStampMaxInterval(LONG *pVal)
{
	if(m_pProfile)
		*pVal = m_pProfile->g_nTsaMaxTsInterval;
    return S_OK;
}

STDMETHODIMP CComDigiDocLib::put_TimeStampMaxInterval(LONG newVal)
{
	if(m_pProfile)
		m_pProfile->g_nTsaMaxTsInterval = newVal;
    return S_OK;
}

STDMETHODIMP CComDigiDocLib::getSetting(BSTR szName, BSTR* pVal)
{
  USES_CONVERSION;
  const char *psVal = NULL;
  char *psName = GetBSTR(szName); 

  if(psName) {
	psVal = ::ConfigItem_lookup(psName);
	if(psVal)
		Convert::copysz2BSTR_withAlloc((char*)psVal, pVal);
  }
  return S_OK;
}

STDMETHODIMP CComDigiDocLib::putSetting(BSTR szName, BSTR szVal)
{
  USES_CONVERSION;
  char *psName = GetBSTR(szName); 
  char *psVal = GetBSTR(szVal); 
  if(psName && psVal)
  ::createOrReplacePrivateConfigItem(NULL, psName, psVal);
  return S_OK;
}





STDMETHODIMP CComDigiDocLib::getSupportedFormats(VARIANT *ret)
{
  const char** pArr = ::getSupportedFormats();
  unsigned int nElems = 0;
  SAFEARRAY* pSArr;
  BSTR HUGEP* pData;

  // detect number of elements
  for(; *pArr != NULL; pArr++, nElems++);

  // initialize SAFEARRAY
  pSArr = SafeArrayCreateVector(VT_BSTR, 0, nElems);
  if (pSArr == NULL) {
    return E_OUTOFMEMORY;
  }

  // init variant
  V_ARRAY(ret) = pSArr;
  V_VT(ret) = (VT_ARRAY | VT_BSTR);

  // copy data to variant
  pArr = ::getSupportedFormats();
  SafeArrayAccessData(pSArr, (void HUGEP**)&pData);
  long lArr[1];
  USES_CONVERSION;
  ddocDebug(1, "getSupportedFormats", "num formats: %d", nElems);
  for(lArr[0] = 0; lArr[0] < (long)nElems; lArr[0]++) {
	  ddocDebug(1, "getSupportedFormats", "format: %d - %s", lArr[0], pArr[lArr[0]]);
    pData[lArr[0]] = SysAllocString(A2OLE(pArr[lArr[0]]));
  }
  SafeArrayUnaccessData(pSArr);

	return S_OK;
}

STDMETHODIMP CComDigiDocLib::extractDataFile(IComSignedDoc* pSignedDoc, BSTR szFileName, BSTR szDataFileName, BSTR szDocId, BSTR szCharset, long* nRetResult)
{
  USES_CONVERSION;
  char *psFileName; 
  char *psDataFileName;
  SignedDoc *pSigDoc = NULL;
  psFileName = GetBSTR(szFileName); 
  psDataFileName = GetBSTR(szDataFileName); 
  if (pSignedDoc != NULL)
    pSigDoc = ((CComSignedDoc*)pSignedDoc)->getSignedDoc();
  *nRetResult = ddocExtractDataFile(pSigDoc, psFileName, psDataFileName, GetBSTR(szDocId), GetBSTR(szCharset));
  return S_OK;
}


STDMETHODIMP CComDigiDocLib::getCertSubjectName(long hCertHandle, BSTR *pCopyTo, long *nRetResult)
{
  DigiDocMemBuf mbuf1, mbuf2;

  mbuf1.pMem = 0;
  mbuf1.nLen = 0;
  mbuf2.pMem = 0;
  mbuf2.nLen = 0;
  *nRetResult = ::ddocCertGetSubjectDN((X509*  )hCertHandle, &mbuf1);
  if (*nRetResult == 0) {
    // allocate as much mem in mbuf2 plus some extra. 
    // Converting to 8 bit makes string shorter so this will be enought
    ddocMemSetLength(&mbuf2, mbuf1.nLen + 10);
    utf82oem((const char*)mbuf1.pMem, (char*)mbuf2.pMem, mbuf2.nLen);
    USES_CONVERSION;
    SetBSTR(pCopyTo, (char*)mbuf2.pMem); //*pCopyTo = SysAllocString(A2OLE(buf));
  }
  ddocMemBuf_free(&mbuf1);
  ddocMemBuf_free(&mbuf2);
  return S_OK;
}

STDMETHODIMP CComDigiDocLib::getCertIssuerName(long hCert, BSTR *pszStoreTo, long *nRetResult)
{
  DigiDocMemBuf mbuf1, mbuf2;

  mbuf1.pMem = 0;
  mbuf1.nLen = 0;
  mbuf2.pMem = 0;
  mbuf2.nLen = 0;
  *nRetResult = ::ddocCertGetIssuerDN((X509*)hCert, &mbuf1);
  if (*nRetResult == 0) {
    // allocate as much mem in mbuf2 plus some extra. 
    // Converting to 8 bit makes string shorter so this will be enought
    ddocMemSetLength(&mbuf2, mbuf1.nLen + 10);
    utf82oem((const char*)mbuf1.pMem, (char*)mbuf2.pMem, mbuf2.nLen);
    USES_CONVERSION;
    SetBSTR(pszStoreTo, (char*)mbuf2.pMem); //*pszStoreTo = SysAllocString(A2OLE(buf));
  }
  ddocMemBuf_free(&mbuf1);
  ddocMemBuf_free(&mbuf2);
  return S_OK;
}

STDMETHODIMP CComDigiDocLib::getCertSerialNumber(long hCert, BSTR *pCopyTo, long *nRetResult)
{
  char buf[X509_NAME_BUF_LEN];
  //AA-Viimase minuti jama
  *nRetResult = ::ReadCertSerialNumber(buf,sizeof(buf),(X509 *)hCert);
  USES_CONVERSION;
  if (*nRetResult == 0)
    *pCopyTo = SysAllocString(A2OLE(buf));
  
//	*nRetResult = ::getCertSerialNumber((void*)hCert, &nTmp);
//  if (*nRetResult == 0)
//    *nCopyTo = nTmp;

	return S_OK;
}

STDMETHODIMP CComDigiDocLib::getCertNotBefore(IComSignedDoc* pUseFormatOf, long hCert, BSTR* szTimeStamp, long *nRetResult)
{
  char buf[X509_NAME_BUF_LEN];
	*nRetResult = ::getCertNotBefore(((CComSignedDoc*)pUseFormatOf)->getSignedDoc(), (X509*)hCert, buf, sizeof(buf));
  if (*nRetResult == ERR_OK) {
    USES_CONVERSION;
    SetBSTR(szTimeStamp,buf); //*szTimeStamp = ::SysAllocString(A2OLE(buf));
  }

	return S_OK;
}

STDMETHODIMP CComDigiDocLib::getCertNotAfter(IComSignedDoc* pUseFormatOf, long hCert, BSTR* szTimeStamp, long *nRetResult)
{
  char buf[X509_NAME_BUF_LEN];
	*nRetResult = ::getCertNotAfter(((CComSignedDoc*)pUseFormatOf)->getSignedDoc(), (X509*)hCert, buf, sizeof(buf));
  if (*nRetResult == 0) {
    USES_CONVERSION;
    SetBSTR(szTimeStamp,buf); //*szTimeStamp = SysAllocString(A2OLE(buf));
  }

	return S_OK;
}

STDMETHODIMP CComDigiDocLib::saveCert(long hCert, BSTR szSaveToFile, long nFormat, long *nRetResult)
{
  USES_CONVERSION;
	*nRetResult = ::saveCert((X509*)hCert, GetBSTR(szSaveToFile), nFormat);

	return S_OK;
}
//Changed by AA 22.09.2003
STDMETHODIMP CComDigiDocLib::isCertValid(long hCert, IComTimestamp* TimeValue, long *nRetResult)
{
	struct tm oTime;
	time_t oTimeVal;
	long lTemp;
	TimeValue->get_year(&lTemp);
	oTime.tm_year = lTemp;
	TimeValue->get_month(&lTemp);
	oTime.tm_mon = lTemp;
	TimeValue->get_day(&lTemp);
	oTime.tm_mday = lTemp;
    //
	oTimeVal = mktime(&oTime);
	*nRetResult = ::isCertValid((X509*)hCert,oTimeVal);

	return S_OK;
}

STDMETHODIMP CComDigiDocLib::isCertSignedBy(long hCert, BSTR szCAFile, long *nRetResult)
{
  USES_CONVERSION;
	*nRetResult = ::isCertSignedBy((X509*)hCert, GetBSTR(szCAFile));

	return S_OK;
}

STDMETHODIMP CComDigiDocLib::isCertSignedByCERT(IComCertSearch *searchCert, IComCertSearch *searchCACert, long *nRetResult)
{
  CertHolderInScope certHolder(*(CComCertSearch*)searchCert);
  CertHolderInScope caCertHolder(*(CComCertSearch*)searchCACert);

  if (certHolder.isAvailable() && caCertHolder.isAvailable()) {
    *nRetResult = ::isCertSignedByCERT(
        certHolder.cert, caCertHolder.cert);
  } else {
    *nRetResult = ERR_CERT_STORE_READ;
  }

	return S_OK;
}

STDMETHODIMP CComDigiDocLib::findIssuerCertificatefromMsStore(IComCertificate *cert, IComCertificate *storeTo)
{
	X509* pCert = ((CComCertificate*)cert)->getX509();
	X509* pIssuerCert = ::findIssuerCertificatefromMsStore(pCert);
	((CComCertificate*)storeTo)->freeStruct_soft();
	if (pIssuerCert)
		((CComCertificate*)storeTo)->takeOwnershipOn(pIssuerCert);

	return S_OK;
}

//AA 19/09/2003
STDMETHODIMP CComDigiDocLib::readCertificateByPKCS12OnlyCert(BSTR szPKCS12File, BSTR szPassword, LONG* hX509Handle)
{
USES_CONVERSION;
X509* pX509 = 0; // VS: 26.01.2010 - initialize with zero
_bstr_t bsFile(szPKCS12File);
_bstr_t bsPsw(szPassword);
if (::Digi_readCertificateByPKCS12OnlyCertHandle((char *)bsFile,(char *)bsPsw,&pX509) == ERR_OK)
  *hX509Handle = (long)pX509;
else
  *hX509Handle = 0;
SysFreeString(bsFile);
SysFreeString(bsPsw);
return S_OK;
}

extern "C" int RunDialogTestLow(void); 
extern "C" int DialogUnitSetTexts(char *psCaption, char *psHeader, char *psOkButton, char *psCancelButton);
extern "C" int DialogUnitSetHeaders(char *psCol1, char *psCol2, char *psCol3, char *psCol4, char *psCol5);
extern "C" int DialogUnitSetToolTips(char *psCommon, char *psListView, char *psOkButton, char *psCancelButton);

STDMETHODIMP CComDigiDocLib::RunDialogTest(long* nRetResult)
{
int iRes = RunDialogTestLow();
*nRetResult = 0;
return S_OK;
}

STDMETHODIMP CComDigiDocLib::dialogUnitSetTexts(BSTR szCaption, BSTR szHeader, BSTR szOkButton, BSTR szCancelButton)
{
	USES_CONVERSION;
DialogUnitSetTexts(OLE2A(szCaption),OLE2A(szHeader),OLE2A(szOkButton),OLE2A(szCancelButton));
return S_OK;
}

STDMETHODIMP CComDigiDocLib::dialogUnitSetHeaders(BSTR szCol1, BSTR szCol2, BSTR szCol3, BSTR szCol4, BSTR szCol5)
{
	USES_CONVERSION;
	DialogUnitSetHeaders(OLE2A(szCol1),OLE2A(szCol2),OLE2A(szCol3),OLE2A(szCol4),OLE2A(szCol5));
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::dialogUnitSetToolTips(BSTR szCommon, BSTR szListView, BSTR szOkButton, BSTR szCancelButton)
{
	USES_CONVERSION;
	DialogUnitSetToolTips(OLE2A(szCommon),OLE2A(szListView),OLE2A(szOkButton),OLE2A(szCancelButton));
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::oem2unicode(BSTR szOem, BSTR* pVal)
{
	char *psBuf;
	int   iLen;
	int   iErr;
	_bstr_t bsTemp(szOem);
	USES_CONVERSION;
	char *psVal = (char *)bsTemp;
	iErr = ::oem2unicode(psVal, &psBuf,&iLen);
	*pVal = SysAllocString(W2OLE((wchar_t *)psBuf));
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::longnum2hex(BSTR longNumber, BSTR* pVal)
{
	char *psBuf;
	int   iLen;
	int   iErr;

	BIGNUM *bn=NULL;
	char* str;

	_bstr_t bsTemp(longNumber);
	char *inputt = bsTemp;

	BN_dec2bn(&bn, inputt);
	str = BN_bn2hex(bn);
	BN_free(bn);
	
	iErr = ::oem2unicode(str, &psBuf,&iLen);
	OPENSSL_free(str);

	*pVal = SysAllocString(W2OLE((wchar_t *)psBuf));
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::unicode2utf8(BSTR szUnicode, BSTR* pVal) 
{
	USES_CONVERSION;
	char *psBuf = NULL;
	int   iLen;
	int   iErr;
    if(szUnicode && *szUnicode) {
	_bstr_t bsTemp(szUnicode);

	
	wchar_t *pswVal = (wchar_t *)bsTemp;
	iErr = ::unicode2utf8((char *)pswVal, &psBuf,&iLen);
	if (!iErr) {
		*pVal = SysAllocString(A2OLE(psBuf));
	} else {
		*pVal = SysAllocString(A2OLE(""));
	}
	if (psBuf != NULL) 
		freeLibMem(psBuf);
	}
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::utf82unicode(BSTR szUtf8, BSTR* pVal)
{
	USES_CONVERSION;
	char *psBuf;
	int   iLen;
	int   iErr;

	if(szUtf8 && *szUtf8) {
		_bstr_t bsTemp(szUtf8);	
		char *psVal = (char *)bsTemp;
		iErr = ::utf82unicode(psVal, &psBuf,&iLen);
		if (!iErr) {
			*pVal = SysAllocString(W2OLE((wchar_t *)psBuf));
		} else {
			*pVal = SysAllocString(W2OLE((wchar_t *)""));
		}
		if (psBuf != NULL) 
			freeLibMem(psBuf);
	}
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::unicode2oem(BSTR szUnicode, BSTR* pVal)
{
	char *psBuf;
	int   iLen;
	int   iErr;

	if(szUnicode && *szUnicode) {
	_bstr_t bsTemp(szUnicode);
	USES_CONVERSION;
	wchar_t *pswVal = (wchar_t *)bsTemp;
	iErr = ::unicode2oem((char *)pswVal, &psBuf,&iLen);
	*pVal = SysAllocString(A2OLE(psBuf));
	if (psBuf != NULL)
		freeLibMem(psBuf);
	}
	return S_OK;
}


STDMETHODIMP CComDigiDocLib::DigiDocErrors(BSTR *retVal)
{
	bstr_t strVal = error_buf.c_str(); 
	*retVal = strVal.copy(); 
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::ClearDigiDocErrors()
{
	error_buf = ""; 
	clearErrors(); 
	return S_OK;
}

STDMETHODIMP CComDigiDocLib::CollectErrors()
{
	char *errorClass[] = {"NO_ERRORS", "TECHNICAL", "USER", "LIBRARY"};
	int err = ERR_OK;
	string sz_ret = ""; 
	char buf[3000];
		
	while(hasUnreadErrors()) {
		ErrorInfo* pErr = getErrorInfo();
		char* pErrStr = getErrorString(pErr->code);			
		if (pErr->code > 0) {
			sprintf(buf,"ERROR: %d - %s; file: %s line: %d; failed condition: %s; error class : %s|",
				   pErr->code, pErrStr, PathFindFileName(pErr->fileName), pErr->line, pErr->assertion, errorClass[getErrorClass(pErr->code)]); 
				error_buf += buf;
		}
	}		
	return S_OK;
}


STDMETHODIMP CComDigiDocLib::SetClientNameVersion(BSTR content){
	string sz_content = _bstr_t(content); 
	setGUIVersion(sz_content.c_str());  	
	return S_OK;
}


