/*

Wrap of DigiCoc library general functions to provide COM-interface
Copyright (C) AS Sertifitseerimiskeskus

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

GNU Lesser General Public Licence is available at
http://www.gnu.org/copyleft/lesser.html

*/
// ComDigiDocLib.h : Declaration of the CComDigiDocLib

#ifndef __COMDIGIDOCLIB_H_
#define __COMDIGIDOCLIB_H_

#include "resource.h"       // main symbols
#include "libdigidoc/DigiDocDefs.h"
#include "libdigidoc/DigiDocLib.h"
#include "libdigidoc/DigiDocSAXParser.h"
#include "libdigidoc/DigiDocCSP.h"
#include "libdigidoc/DigiDocGlobals.h"

/////////////////////////////////////////////////////////////////////////////
// CComDigiDocLib

#include "ComSignedDoc.h"
#include "ComSignatureInfo.h"
#include "ComDocInfo.h"
#include "ComNotaryInfo.h"
//#include "ComSignatureProdPlace.h"
#include "ComCertSearch.h"

class ATL_NO_VTABLE CComDigiDocLib : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CComDigiDocLib, &CLSID_ComDigiDocLib>,
	public IDispatchImpl<IComDigiDocLib, &IID_IComDigiDocLib, &LIBID_DIGIDOCLIBCOMLib>
{
public:
	CComDigiDocLib();
	~CComDigiDocLib();

DECLARE_REGISTRY_RESOURCEID(IDR_COMDIGIDOCLIB)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CComDigiDocLib)
	COM_INTERFACE_ENTRY(IComDigiDocLib)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

// IComDigiDocLib
public:
	STDMETHOD(SetClientNameVersion)(BSTR content);
	STDMETHOD(getComVersion)(BSTR *ret);



	STDMETHOD(CollectErrors)();
	STDMETHOD(ClearDigiDocErrors)();
	string error_buf;
	STDMETHOD(DigiDocErrors)(BSTR *retVal);
	STDMETHOD(findIssuerCertificatefromMsStore)(/*[in]*/ IComCertificate* cert, /*[in]*/ IComCertificate* storeTo);
	STDMETHOD(isCertSignedByCERT)(/*[in]*/ IComCertSearch* searchCert, /*[in]*/ IComCertSearch* searchCACert, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(isCertSignedBy)(/*[in]*/ long hCert, /*[in]*/ BSTR szCAFile, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(isCertValid)(/*[in]*/ long hCert, /*[in]*/ IComTimestamp* TimeValue, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(saveCert)(/*[in]*/ long hCert, /*[in]*/ BSTR szSaveToFile, /*[in]*/ long nFormat, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(getCertNotAfter)(/*[in]*/ IComSignedDoc* pUseFormatOf, /*[in]*/ long hCert, /*[out]*/ BSTR* szTimeStamp, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(getCertNotBefore)(/*[in]*/ IComSignedDoc* pUseFormatOf, /*[in]*/ long hCert, /*[out]*/ BSTR* szTimeStamp, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(getCertSerialNumber)(/*[in]*/ long hCert, /*[out]*/ BSTR* pCopyTo, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(getCertIssuerName)(/*[in]*/ long hCert, /*[out]*/ BSTR* pszStoreTo, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(getCertSubjectName)(/*[in]*/ long hCertHandle, /*[out]*/ BSTR* pCopyTo, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(extractDataFile)(/*[in]*/ IComSignedDoc* pSignedDoc,/*[in]*/ BSTR szFileName, /*[in]*/ BSTR szDataFileName, /*[in]*/ BSTR szDocId, /*[in]*/ BSTR szCharset, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(getSupportedFormats)(/*[out, retval]*/ VARIANT* ret);
	STDMETHOD(getLibVersion)(/*[out, retval]*/ BSTR* ret);
	STDMETHOD(getLibName)(/*[out, retval]*/ BSTR* ret);
	STDMETHOD(readCertificateByPKCS12OnlyCert)(BSTR szPKCS12File, BSTR szPassword, LONG* hX509Handle);
	STDMETHOD(RunDialogTest)( /*[out, retval]*/ long* nRetResult);
	STDMETHOD(dialogUnitSetTexts)(BSTR szCaption, BSTR szHeader, BSTR szOkButton, BSTR szCancelButton);
	STDMETHOD(dialogUnitSetHeaders)(BSTR szCol1, BSTR szCol2, BSTR szCol3, BSTR szCol4, BSTR szCol5);
	STDMETHOD(dialogUnitSetToolTips)(BSTR szCommon, BSTR szListView, BSTR szOkButton, BSTR szCancelButton);
	STDMETHOD(oem2unicode)(BSTR szOem, BSTR* pVal);
	STDMETHOD(longnum2hex)(BSTR longNumber, BSTR* pVal);
	STDMETHOD(unicode2utf8)(BSTR szUnicode, BSTR* pVal);
	STDMETHOD(utf82unicode)(BSTR szUtf8, BSTR* pVal);
	STDMETHOD(unicode2oem)(BSTR szUnicode, BSTR* pVal);

	// WITH_TS update
	STDMETHOD(get_TsaUrl)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_TsaUrl)(/*[in]*/ BSTR newVal);	

	STDMETHOD(get_AddTimeStamp)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_AddTimeStamp)(/*[in]*/ VARIANT_BOOL newVal);

	STDMETHOD(get_TimeStampMaxInterval)(/*[out, retval]*/ LONG *pVal);
	STDMETHOD(put_TimeStampMaxInterval)(/*[in]*/ LONG newVal);	

	STDMETHOD(get_DebugFilePath)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_DebugFilePath)(/*[in]*/ BSTR newVal);

	STDMETHOD(get_DebugLevel)(/*[out, retval]*/ LONG *pVal);
	STDMETHOD(put_DebugLevel)(/*[in]*/ LONG newVal);

	STDMETHOD(get_AddSigAndRefsTimeStamp)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_AddSigAndRefsTimeStamp)(/*[in]*/ VARIANT_BOOL newVal);

	STDMETHOD(getSetting)(/*[in]*/ BSTR szName, /*[out, retval]*/ BSTR *pVal);
	STDMETHOD(putSetting)(/*[in]*/ BSTR szName, /*[in]*/ BSTR szValue);

private:
	TSAProfile * m_pProfile;
};

#endif //__COMDIGIDOCLIB_H_
