// ComSignatureInfo.cpp : Implementation of CComSignatureInfo
#include "stdafx.h"
#include "DigiDocLibCOM_h.h"
#include "libdigidoc/DigiDocVerify.h"
#include "libdigidoc/DigiDocPKCS11.h"
#include "libdigidoc/DigiDocDebug.h"

#include "ComSignatureInfo.h"
#include "Converters.h"

/////////////////////////////////////////////////////////////////////////////
// CComSignatureInfo


//////////////////////////////////////////////////////////////
//        properties
//////////////////////////////////////////////////////////////

STDMETHODIMP CComSignatureInfo::get_szId(BSTR *pVal)
{
	if(m_pSignatureInfo && m_pSignatureInfo->szId)
		Convert::out_retval_sz2BSTR(m_pSignatureInfo->szId, pVal);
	else
		Convert::out_retval_sz2BSTR("", pVal);
	return S_OK;
}


STDMETHODIMP CComSignatureInfo::get_nDocs(long *pVal)
{
	if(m_pSignatureInfo)
		*pVal = m_pSignatureInfo->nDocs;
	else
		*pVal = 0;
	return S_OK;
}

STDMETHODIMP CComSignatureInfo::get_szTimeStamp(BSTR *pVal)
{
	if(m_pSignatureInfo)
		Convert::out_retval_sz2BSTR(m_pSignatureInfo->szTimeStamp, pVal);
	else
		Convert::out_retval_sz2BSTR("", pVal);
	return S_OK;
}

STDMETHODIMP CComSignatureInfo::put_szTimeStamp(BSTR newVal)
{
	if(m_pSignatureInfo)
		Convert::in_BSTR2sz(newVal, &(m_pSignatureInfo->szTimeStamp));

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::get_nSigPropDigestLen(long *pVal)
{
	DigiDocMemBuf* pMBuf = 0;
	*pVal = 0;
	if(m_pSignatureInfo) {
		pMBuf = ddocSigInfo_GetSigPropDigest(m_pSignatureInfo);
		if(pMBuf)
			*pVal = pMBuf->nLen;
	}
    return S_OK;
}

STDMETHODIMP CComSignatureInfo::get_szSigType(BSTR *pVal)
{
	if(m_pSignatureInfo && m_pSignatureInfo->pSigValue)
		Convert::out_retval_sz2BSTR((char*)ddocSignatureValue_GetType(m_pSignatureInfo->pSigValue), pVal);
	else
		Convert::out_retval_sz2BSTR("", pVal);
	return S_OK;
}

STDMETHODIMP CComSignatureInfo::put_szSigType(BSTR newVal)
{
	//if(getSignatureInfo()->pSigValue)
	//Convert::in_BSTR2sz(newVal, &(getSignatureInfo()->szSigType));
	// don't do it!
	return S_OK;
}


STDMETHODIMP CComSignatureInfo::get_nSigLen(long *pVal)
{
	DigiDocMemBuf* pMBuf = 0;
	*pVal = 0;
	if(m_pSignatureInfo && m_pSignatureInfo->pSigValue) {
		pMBuf = ddocSignatureValue_GetSignatureValue(m_pSignatureInfo->pSigValue);
		if(pMBuf)
			*pVal = pMBuf->nLen;
	}
	return S_OK;
}

STDMETHODIMP CComSignatureInfo::get_nIssuerSerial(BSTR *pVal)
{
	if(m_pSignatureInfo && ddocSigInfo_GetSignersCert_IssuerSerial(m_pSignatureInfo))
		Convert::out_retval_sz2BSTR((char*)ddocSigInfo_GetSignersCert_IssuerSerial(m_pSignatureInfo), pVal);
	else
		Convert::out_retval_sz2BSTR("", pVal);
	return S_OK;
}


STDMETHODIMP CComSignatureInfo::get_nCertDigestLen(long *pVal)
{
	DigiDocMemBuf* pMBuf = 0;
	*pVal = 0;
	if(m_pSignatureInfo) {
		pMBuf = ddocSigInfo_GetSignersCert_DigestValue(m_pSignatureInfo);
		if(pMBuf)
			*pVal = pMBuf->nLen;
	}
	return S_OK;
}

STDMETHODIMP CComSignatureInfo::get_verifiedByWrongDataFileHash(long *pVal)
{
	if(m_pSignatureInfo)
		*pVal = verifiedByWrongDataFileHash(m_pSignatureInfo);
	else
		*pVal = 0;
	return S_OK;
}

//////////////////////////////////////////////////////////////
//        methods
//////////////////////////////////////////////////////////////

STDMETHODIMP CComSignatureInfo::verifySigPropDigest(IComSignedDoc* pSigDoc,BSTR szDataFile, long *nRetResult)
{
  USES_CONVERSION;


  *nRetResult = 0; //::verifySigDocSigPropDigest(((CComSignedDoc*)pSigDoc)->getSignedDoc(),getSignatureInfo(), GetBSTR(szDataFile));
  
  return S_OK;
}

STDMETHODIMP CComSignatureInfo::addDocInfo(BSTR szDocId, BSTR szDigType, 
    VARIANT baDigest, long nDigLen, VARIANT baMimeDig, long nMimeDigLen, IComDocInfo* pStoreTo)
{
  SAFEARRAY* pArrDig = V_ARRAY(&baDigest);
  SAFEARRAY* pArrMDig = V_ARRAY(&baMimeDig);
  byte HUGEP* pDataDig;
  byte HUGEP* pDataMDig;

  SafeArrayAccessData(pArrDig, (void HUGEP**)&pDataDig);
  SafeArrayAccessData(pArrMDig, (void HUGEP**)&pDataMDig);
  USES_CONVERSION;
  DocInfo* pDI = NULL;
  ::addDocInfo(&pDI, m_pSignatureInfo, GetBSTR(szDocId), GetBSTR(szDigType), 
      pDataDig, nDigLen, pDataMDig, nMimeDigLen);
  SafeArrayUnaccessData(pArrMDig);
  SafeArrayUnaccessData(pArrDig);

  if (pDI != (DocInfo*)0)
    ((CComDocInfo*)pStoreTo)->shiftStructPointer(pDI);

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::calculateSignatureWithEstID(IComSignedDoc* pSigDoc,long nSlot, BSTR szPasswd, long *nRetResult)
{
	// test
	::createOrReplacePrivateConfigItem(NULL, "DIGIDOC_DEFAULT_DRIVER", "1");
	::createOrReplacePrivateConfigItem(NULL, "DIGIDOC_DRIVER_1_FILE", "opensc-pkcs11.dll");
	::createOrReplacePrivateConfigItem(NULL, "DEBUG_LEVEL", "3");
	::createOrReplacePrivateConfigItem(NULL, "DEBUG_FILE", "y:\\digidoc.log");
  if(((CComSignedDoc*)pSigDoc)->getSignedDoc() && ::hasSignatureWithWrongDataFileHash(((CComSignedDoc*)pSigDoc)->getSignedDoc())) {
        ::ddocDebug(1, "createSignatureInfo", "Cannot add signature in ddoc with invalid DataFile hashes!");
        *nRetResult = ERR_FILE_WRITE;
        return S_OK;
  }
  if(m_pSignatureInfo)
    *nRetResult = ::calculateSignatureWithEstID(((CComSignedDoc*)pSigDoc)->getSignedDoc(), m_pSignatureInfo, nSlot, GetBSTR(szPasswd));
  return S_OK;
}

STDMETHODIMP CComSignatureInfo::calculateSignatureWithCSPEstID(IComSignedDoc* pSigDoc, long fByKeyContainer, BSTR szPin, long *pRetErr)
{
    if(((CComSignedDoc*)pSigDoc)->getSignedDoc() && ::hasSignatureWithWrongDataFileHash(((CComSignedDoc*)pSigDoc)->getSignedDoc())) {
        ::ddocDebug(1, "createSignatureInfo", "Cannot add signature in ddoc with invalid DataFile hashes!");
        *pRetErr = ERR_FILE_WRITE;
        return S_OK;
    }
	if(m_pSignatureInfo)
		*pRetErr = ::calculateSigInfoSignatureWithCSPEstID(((CComSignedDoc*)pSigDoc)->getSignedDoc(),m_pSignatureInfo,fByKeyContainer, GetBSTR(szPin));
	return S_OK;
}

STDMETHODIMP CComSignatureInfo::addSignerRole(VARIANT tfCertified, BSTR szRole, VARIANT tfEncode)
{
	int nCertified = (V_BOOL(&tfCertified) ? 1 : 0);
	int nEncode = (V_BOOL(&tfEncode) ? 1 : 0);
	const char* pRoleAsUTF8 = GetBSTR(szRole);
	int nRoleLen = strlen(pRoleAsUTF8) + 1;

  USES_CONVERSION;
  if(m_pSignatureInfo)
  ::addSignerRole(m_pSignatureInfo, nCertified, pRoleAsUTF8, nRoleLen, nEncode);

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::countSignerRoles(VARIANT tfCertified, long *nRetCnt)
{

	int nCertified = (V_BOOL(&tfCertified) ? 1 : 0);
	if(m_pSignatureInfo)
		*nRetCnt = ::getCountOfSignerRoles(m_pSignatureInfo, nCertified);
	else
		*nRetCnt = 0;

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::getSignerRole(VARIANT tfCertified, long nIndex, BSTR *pRetRole)
{
	const char* pPtr = 0;
	int nCertified = (V_BOOL(&tfCertified) ? 1 : 0);
	if(m_pSignatureInfo) {
		pPtr = ::getSignerRole(m_pSignatureInfo, nCertified, nIndex);
		if(pPtr) 
			Convert::out_retval_sz2BSTR((char*)pPtr, pRetRole);
	}
	return S_OK;
}

STDMETHODIMP CComSignatureInfo::createXMLSignedProperties(IComSignedDoc* pSigDoc,BSTR *pRetProperties)
{
  USES_CONVERSION;
  

  //char* pTmp = ::createXMLSignedProperties(((CComSignedDoc*)pSigDoc)->getSignedDoc(),getSignatureInfo(), 0);
	SetBSTR(pRetProperties, ""); //*pRetProperties = SysAllocString(A2OLE(pTmp));
  //free(pTmp);

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::setSignatureCertFile(BSTR szCertFile, long *nRetResult)
{
  USES_CONVERSION;
  if(m_pSignatureInfo)
	*nRetResult = ::setSignatureCertFile(m_pSignatureInfo, GetBSTR(szCertFile));

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::calculateSignedPropertiesDigest(IComSignedDoc* pSigDoc,long *nRetResult)
{
	if(m_pSignatureInfo)
	*nRetResult = ::calculateSignedPropertiesDigest(((CComSignedDoc*)pSigDoc)->getSignedDoc(),m_pSignatureInfo);

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::getSignCertData(long *hRetCertHandle)
{
   if(m_pSignatureInfo)
	*hRetCertHandle = (long)::getSignCertData(m_pSignatureInfo);

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::getDocInfo(long nIndex, IComDocInfo* pStoreTo)
{
  DocInfo* pDI = 0;
  if(m_pSignatureInfo)
	pDI = ::getDocInfo(m_pSignatureInfo, nIndex);
  if (pDI != (DocInfo*)0) {
    ((CComDocInfo*)pStoreTo)->shiftStructPointer(pDI);
  }

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::getDocInfoWithId(BSTR szId, IComDocInfo* pStoreTo)
{
  USES_CONVERSION;
	DocInfo* pDI = 0;
  if(m_pSignatureInfo)
	pDI = ::getDocInfoWithId(m_pSignatureInfo, GetBSTR(szId));
  if (pDI != (DocInfo*)0) {
    ((CComDocInfo*)pStoreTo)->shiftStructPointer(pDI);
  }

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::calculateSigInfoSignature(IComSignedDoc* pSigDoc,long nSigType, BSTR szKeyFile, 
    BSTR szPasswd, BSTR szCertfile, long* nRetErr)
{
  USES_CONVERSION;
  if(m_pSignatureInfo)
	*nRetErr = ::calculateSigInfoSignature(((CComSignedDoc*)pSigDoc)->getSignedDoc(),m_pSignatureInfo, nSigType, 
      GetBSTR(szKeyFile), GetBSTR(szPasswd), GetBSTR(szCertfile));

	return S_OK;
}

STDMETHODIMP CComSignatureInfo::verifySigCert(long *nRetResult)
{
  if(m_pSignatureInfo)
	*nRetResult = ::verifySigCert(m_pSignatureInfo);

  return S_OK;
}

STDMETHODIMP CComSignatureInfo::setSignatureProductionPlace(BSTR szCity, BSTR szState, BSTR szZipCode, BSTR szCountry)
{
  USES_CONVERSION;

  char *p1 = (szCity ? GetBSTR(szCity) : NULL);
  char *p2 = (szState ? GetBSTR(szState) : NULL);
  char *p3 = (szZipCode ? GetBSTR(szZipCode) : NULL);
  char *p4 = (szCountry ? GetBSTR(szCountry) : NULL);
  if(m_pSignatureInfo)
	::setSignatureProductionPlace(m_pSignatureInfo, p1, p2, p3, p4); 
  return S_OK;
}

STDMETHODIMP CComSignatureInfo::getSignatureProductionPlace(BSTR* pszCity, BSTR* pszState, BSTR* pszZipCode, BSTR* pszCountry)
{
  USES_CONVERSION;
  if(m_pSignatureInfo) {
	  SetBSTR(pszCity, (m_pSignatureInfo->sigProdPlace.szCity ? m_pSignatureInfo->sigProdPlace.szCity : ""));
	  SetBSTR(pszState, (m_pSignatureInfo->sigProdPlace.szStateOrProvince ? m_pSignatureInfo->sigProdPlace.szStateOrProvince : ""));
	  SetBSTR(pszZipCode, (m_pSignatureInfo->sigProdPlace.szPostalCode ? m_pSignatureInfo->sigProdPlace.szPostalCode : ""));
	  SetBSTR(pszCountry, (m_pSignatureInfo->sigProdPlace.szCountryName ? m_pSignatureInfo->sigProdPlace.szCountryName : ""));
  }
  return S_OK;
}

////////////////////////////////////////////////////////////////////
//    helpers
////////////////////////////////////////////////////////////////////

CComSignatureInfo::CComSignatureInfo() {
	m_pSignatureInfo = NULL;
	m_pSigDoc = NULL;
}

CComSignatureInfo::~CComSignatureInfo() {
	// actually deleted probably by lib
	//if(m_pSignatureInfo) ::SignatureInfo_free(m_pSignatureInfo);
}

SignatureInfo* CComSignatureInfo::getSignatureInfo() {
  return m_pSignatureInfo;
}

void CComSignatureInfo::shiftStructPointer(SignatureInfo* pNew) {
  // actually deleted probably by lib
  //if(m_pSignatureInfo) ::SignatureInfo_free(m_pSignatureInfo);
  m_pSignatureInfo = pNew;
}

void CComSignatureInfo::setSignedDoc(SignedDoc* pSigDoc) {
  m_pSigDoc = pSigDoc;
}

