/*

Wrap of DigiCoc library SignatureInfo structure to provide COM-interface
Copyright (C) AS Sertifitseerimiskeskus

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

GNU Lesser General Public Licence is available at
http://www.gnu.org/copyleft/lesser.html

*/
// ComSignatureInfo.h : Declaration of the CComSignatureInfo

#ifndef __COMSIGNATUREINFO_H_
#define __COMSIGNATUREINFO_H_

#include "resource.h"       // main symbols

/////////////////////////////////////////////////////////////////////////////
// CComSignatureInfo

#include "libdigidoc/DigiDocDefs.h"
#include "libdigidoc/DigiDocLib.h"
#include "libdigidoc/DigiDocObj.h"
#include "libdigidoc/DigiDocGen.h"
#include "libdigidoc/DigiDocVerify.h"
//#include "ComSignerRole.h"
#include "ComDocInfo.h"
#include "ComSignedDoc.h"
//#include "ComSignatureProdPlace.h"

class ATL_NO_VTABLE CComSignatureInfo : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CComSignatureInfo, &CLSID_ComSignatureInfo>,
	public IDispatchImpl<IComSignatureInfo, &IID_IComSignatureInfo, &LIBID_DIGIDOCLIBCOMLib>
{
public:
	CComSignatureInfo();
	~CComSignatureInfo();

DECLARE_REGISTRY_RESOURCEID(IDR_COMSIGNATUREINFO)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CComSignatureInfo)
	COM_INTERFACE_ENTRY(IComSignatureInfo)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

// IComSignatureInfo
public:
	STDMETHOD(getSignatureProductionPlace)(/*[out]*/ BSTR* pszCity, /*[out]*/ BSTR* pszState, /*[out]*/ BSTR* pszZipCode, /*[out]*/ BSTR* pszCountry);
	STDMETHOD(setSignatureProductionPlace)(/*[in]*/ BSTR szCity, /*[in]*/ BSTR szState, /*[in]*/ BSTR szZipCode, /*[in]*/ BSTR szCountry);
	//STDMETHOD(getSigProdPlace)(/*[in]*/ IComSignatureProdPlace *pStoreTo);
	STDMETHOD(verifySigCert)(/*[out, retval]*/ long* nRetResult);
	STDMETHOD(calculateSigInfoSignature)(/*[in]*/ IComSignedDoc* pSigDoc,/*[in]*/ long nSigType, /*[in]*/ BSTR szKeyFile, /*[in]*/ BSTR szPasswd, /*[in]*/ BSTR szCertfile, /*[out, retval]*/ long* nRetErr);
	STDMETHOD(getDocInfoWithId)(/*[in]*/ BSTR szId, /*[in]*/ IComDocInfo* pStoreTo);
	STDMETHOD(getDocInfo)(/*[in]*/ long nIndex, /*[in]*/ IComDocInfo* pStoreTo);
	STDMETHOD(getSignCertData)(/*[out, retval]*/ long* hRetCertHandle);
	STDMETHOD(calculateSignedPropertiesDigest)(/*[in]*/ IComSignedDoc* pSigDoc,/*[out, retval]*/ long* nRetResult);
	STDMETHOD(setSignatureCertFile)(/*[in]*/ BSTR szCertFile, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(createXMLSignedProperties)(/*[in]*/ IComSignedDoc* pSigDoc,/*[out, retval]*/ BSTR* pRetProperties);
	//STDMETHOD(getSignerRoleObject)(/*[in]*/ IComSignerRole* pStoreTo);
	STDMETHOD(getSignerRole)(/*[in]*/ VARIANT tfCertified, /*[in]*/ long nIndex, /*[out, retval]*/ BSTR* pRetRole);
	STDMETHOD(countSignerRoles)(/*[in]*/ VARIANT tfCertified, /*[out, retval]*/ long* nRetCnt);
	STDMETHOD(addSignerRole)(/*[in]*/ VARIANT tfCertified, /*[in]*/ BSTR szRole, /*[in]*/ VARIANT tfEncode);
	STDMETHOD(calculateSignatureWithCSPEstID)(/*[in]*/ IComSignedDoc* pSigDoc,/*in*/long fByKeyContainer, /*[in]*/ BSTR szPin,/*[out, retval]*/ long* pRetErr);
	STDMETHOD(calculateSignatureWithEstID)(/*[in]*/ IComSignedDoc* pSigDoc,/*[in]*/ long nSlot, /*[in]*/ BSTR szPasswd, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(addDocInfo)(/*[in]*/ BSTR szDocId, /*[in]*/ BSTR szDigType, /*[in]*/ VARIANT baDigest, /*[in]*/ long nDigLen, /*[in]*/ VARIANT baMimeDig, /*[in]*/ long nMimeDigLen, /*[in]*/ IComDocInfo* pStoreTo);
	STDMETHOD(verifySigPropDigest)(/*[in]*/ IComSignedDoc* pSigDoc,/*[in]*/ BSTR szDataFile, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(get_nCertDigestLen)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_nIssuerSerial)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(get_nSigLen)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_szSigType)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_szSigType)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_nSigPropDigestLen)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_szTimeStamp)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_szTimeStamp)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_nDocs)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_szId)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(get_verifiedByWrongDataFileHash)(/*[out, retval]*/ long *pVal);

	SignatureInfo* getSignatureInfo();

	void shiftStructPointer(SignatureInfo* pNew);
	void setSignedDoc(SignedDoc* pSigDoc);

protected:
	SignedDoc* m_pSigDoc;
	SignatureInfo* m_pSignatureInfo;
};

#endif //__COMSIGNATUREINFO_H_
