// ComSignedDoc.cpp : Implementation of CComSignedDoc
#include "stdafx.h"
#include "DigiDocLibCOM_h.h"
#include "libdigidoc/DigiDocGlobals.h"
#include "libdigidoc/DigiDocDebug.h"
#include "libdigidoc/DigiDocConvert.h"
#include "ComSignedDoc.h"
#include "Converters.h"
#include "ComCertArray.h"
/////////////////////////////////////////////////////////////////////////////
// CComSignedDoc


//////////////////////////////////////////////////////////////
//        properties
//////////////////////////////////////////////////////////////

STDMETHODIMP CComSignedDoc::get_szFormat(BSTR *pVal)
{
	if(m_pSignedDoc)
		Convert::out_retval_sz2BSTR(m_pSignedDoc->szFormat, pVal);
	else
		Convert::out_retval_sz2BSTR("", pVal);
	return S_OK;
}

STDMETHODIMP CComSignedDoc::get_szFormatVer(BSTR *pVal)
{
	if(m_pSignedDoc)
		Convert::out_retval_sz2BSTR(m_pSignedDoc->szFormatVer, pVal);
	else
		Convert::out_retval_sz2BSTR("", pVal);
	return S_OK;
}

STDMETHODIMP CComSignedDoc::get_nDataFiles(long *pVal)
{
	*pVal = 0;
	if(m_pSignedDoc)
		*pVal = m_pSignedDoc->nDataFiles;
	return S_OK;
}


STDMETHODIMP CComSignedDoc::get_nSignatures(long *pVal)
{
	*pVal = 0;
	if(m_pSignedDoc)
		*pVal = m_pSignedDoc->nSignatures;
	return S_OK;
}


STDMETHODIMP CComSignedDoc::get_nNotaries(long *pVal)
{
	*pVal = 0;
	if(m_pSignedDoc)
		*pVal = getCountOfNotaryInfos(m_pSignedDoc);

	return S_OK;
}


//////////////////////////////////////////////////////////////
//        methods
//////////////////////////////////////////////////////////////

STDMETHODIMP CComSignedDoc::initialize(BSTR szFormat, BSTR szVersion)
{
	USES_CONVERSION;
	SignedDoc* pNew = NULL;
  
	//initConfigStore(NULL);
	::SignedDoc_new(&pNew,GetBSTR(szFormat), GetBSTR(szVersion));
	if (pNew != (SignedDoc*)0) {
		shiftStructPointer(pNew);
	}
	return S_OK;
}

STDMETHODIMP CComSignedDoc::createDataFile(BSTR szFileName, BSTR szContentType, BSTR szMimeType, long nSize, VARIANT baDigest, long nDigLen, BSTR szDigType, BSTR szCharset, IComDataFile *pStoreTo)
{
	SAFEARRAY* pArr;
	byte HUGEP* pData;
	USES_CONVERSION;
	
	if(!m_pSignedDoc)
		return S_OK;
    if(::hasSignatureWithWrongDataFileHash(m_pSignedDoc)) {
        ddocDebug(1, "createDataFile", "Cannot add signature in ddoc with invalid DataFile hashes!");
        return S_OK;
    }
	// analyze digest array
	if ((V_VT(&baDigest) != VT_EMPTY)
		  && (V_VT(&baDigest) == VT_ARRAY)) {
		pArr = V_ARRAY(&baDigest);
		if ((SafeArrayGetDim(pArr) <= 0)
			|| (SafeArrayAccessData(pArr, (void HUGEP**)&pData) != S_OK))
	  pData = NULL;
	} else {
		pData = NULL;
	}


    DataFile* pDataFile = NULL;

	char *p1 = GetBSTR(szFileName);
	char *p2 = GetBSTR(szContentType);
	char *p3 = GetBSTR(szMimeType);
	char *p4 = GetBSTR(szDigType);
	char *p5 = GetBSTR(szCharset);

	::DataFile_new(&pDataFile, m_pSignedDoc, NULL, 
					p1, p2, p3, nSize, pData, nDigLen, p4, p5);

	if (pData)
	    SafeArrayUnaccessData(pArr);

	if (pDataFile != (DataFile*)0) {
		CComDataFile* pComDataFile =  (CComDataFile*)pStoreTo;
		pComDataFile->shiftStructPointer(pDataFile);
		pComDataFile->setSignedDoc(m_pSignedDoc); 
	}
	return S_OK;
}

STDMETHODIMP CComSignedDoc::getDataFile(long nIndex, IComDataFile* pStoreTo)
{
	DataFile* pDataFile = 0;
	if(m_pSignedDoc)
		pDataFile = ::getDataFile(m_pSignedDoc, nIndex);
	if (pDataFile != (DataFile*)0) {
		CComDataFile* pComDataFile =  (CComDataFile*)pStoreTo;
		pComDataFile->shiftStructPointer(pDataFile);
		pComDataFile->setSignedDoc(m_pSignedDoc); 
	}
	return S_OK;
}

STDMETHODIMP CComSignedDoc::getDataFileWithId(BSTR szId, IComDataFile *pStoreTo)
{
  USES_CONVERSION;
  DataFile* pDataFile = 0;
  if(m_pSignedDoc)
	pDataFile = ::getDataFileWithId(m_pSignedDoc, GetBSTR(szId));
  if (pDataFile != (DataFile*)0) {
    CComDataFile* pComDataFile =  (CComDataFile*)pStoreTo;
    pComDataFile->shiftStructPointer(pDataFile);
	pComDataFile->setSignedDoc(getSignedDoc());  
  }
	return S_OK;
}

STDMETHODIMP CComSignedDoc::calculateDataFileSizeAndDigest(BSTR szId, BSTR szFileName, long digType, long* nRetResult)
{
  USES_CONVERSION;
  if(m_pSignedDoc)
	(*nRetResult) = ::calculateDataFileSizeAndDigest(m_pSignedDoc, GetBSTR(szId), GetBSTR(szFileName), digType);
  return S_OK;
}


//////////////////////////////////////////////////////////
//    SignatureInfo methods
STDMETHODIMP CComSignedDoc::createSignatureInfo(IComSignatureInfo* pStoreTo)
{
  SignatureInfo* pSigInfo = NULL;

  /*if(m_pSignedDoc && ::hasSignatureWithWrongDataFileHash(m_pSignedDoc)) {
        ddocDebug(1, "createSignatureInfo", "Cannot add signature in ddoc with invalid DataFile hashes!");
        return S_OK;
  }*/
  if(m_pSignedDoc)
	::SignatureInfo_new(&pSigInfo, m_pSignedDoc, NULL);
  if (pSigInfo != (SignatureInfo*)0) {
    ((CComSignatureInfo*)pStoreTo)->shiftStructPointer(pSigInfo);
	((CComSignatureInfo*)pStoreTo)->setSignedDoc(m_pSignedDoc);
  }
  return S_OK;
}

STDMETHODIMP CComSignedDoc::getSignature(long nIndex, IComSignatureInfo *pStoreTo)
{
	SignatureInfo* pSigInfo = 0;
	if(m_pSignedDoc)
		pSigInfo = ::getSignature(m_pSignedDoc, nIndex);
    if (pSigInfo != (SignatureInfo*)0) {
		((CComSignatureInfo*)pStoreTo)->shiftStructPointer(pSigInfo);
		((CComSignatureInfo*)pStoreTo)->setSignedDoc(m_pSignedDoc);
	}
	return S_OK;
}

STDMETHODIMP CComSignedDoc::getSignatureWithId(BSTR szId, IComSignatureInfo *pStoreTo)
{
	USES_CONVERSION;
	SignatureInfo* pSigInfo = 0;
	if(m_pSignedDoc)
		pSigInfo = ::getSignatureWithId(m_pSignedDoc, GetBSTR(szId));
	if (pSigInfo != (SignatureInfo*)0) {
		((CComSignatureInfo*)pStoreTo)->shiftStructPointer(pSigInfo);
		((CComSignatureInfo*)pStoreTo)->setSignedDoc(m_pSignedDoc);
	}
	return S_OK;
}

STDMETHODIMP CComSignedDoc::addAllDocInfos(IComSignatureInfo *pSigInfo)
{
	if(m_pSignedDoc)
		::addAllDocInfos(m_pSignedDoc, ((CComSignatureInfo*)pSigInfo)->getSignatureInfo());
	return S_OK;
}

STDMETHODIMP CComSignedDoc::getNotaryInfo(long nIndex, IComNotaryInfo *pStoreTo)
{
	NotaryInfo* pNotInfo = 0;
	if(m_pSignedDoc)
		pNotInfo = ::getNotaryInfo(m_pSignedDoc, nIndex);
	if (pNotInfo != (NotaryInfo*)0){
		((CComNotaryInfo*)pStoreTo)->shiftStructPointer(pNotInfo);
		SignatureInfo *pSigInfo = ddocGetSignatureForNotary(m_pSignedDoc, pNotInfo);
		if(pSigInfo)
			((CComNotaryInfo*)pStoreTo)->setSignatureInfo(pSigInfo);
	}

	return S_OK;
}

STDMETHODIMP CComSignedDoc::getNotaryWithId(BSTR szId, IComNotaryInfo *pStoreTo)
{
  USES_CONVERSION;
  NotaryInfo* pNotInfo = 0;
  if(m_pSignedDoc)
	pNotInfo = ::getNotaryWithId(m_pSignedDoc, GetBSTR(szId));
  if (pNotInfo != (NotaryInfo*)0){
    ((CComNotaryInfo*)pStoreTo)->shiftStructPointer(pNotInfo);
	SignatureInfo *pSigInfo = ddocGetSignatureForNotary(m_pSignedDoc, pNotInfo);
	if(pSigInfo)
		((CComNotaryInfo*)pStoreTo)->setSignatureInfo(pSigInfo);
  }
  return S_OK;
}

STDMETHODIMP CComSignedDoc::getNotaryWithSigId(BSTR szSigId, IComNotaryInfo *pStoreTo)
{
  USES_CONVERSION;
  NotaryInfo* pNotInfo = 0;
  if(m_pSignedDoc)
	pNotInfo = ::getNotaryWithSigId(m_pSignedDoc, GetBSTR(szSigId));
  if (pNotInfo != (NotaryInfo*)0){
    ((CComNotaryInfo*)pStoreTo)->shiftStructPointer(pNotInfo);
	SignatureInfo *pSigInfo = ddocGetSignatureForNotary(m_pSignedDoc, pNotInfo);
	if(pSigInfo)
		((CComNotaryInfo*)pStoreTo)->setSignatureInfo(pSigInfo);
  }
  return S_OK;
}

STDMETHODIMP CComSignedDoc::createNotaryInfo(IComSignatureInfo *pSigInfo, IComNotaryInfo *pStoreTo)
{
  NotaryInfo* pNotInfo = NULL;
  if(m_pSignedDoc)
  ::NotaryInfo_new(&pNotInfo, m_pSignedDoc, 
      ((CComSignatureInfo*)pSigInfo)->getSignatureInfo());
  if (pNotInfo != (NotaryInfo*)0) {
    ((CComNotaryInfo*)pStoreTo)->shiftStructPointer(pNotInfo);
	((CComNotaryInfo*)pStoreTo)->setSignatureInfo(((CComSignatureInfo*)pSigInfo)->getSignatureInfo());
  }

	return S_OK;
}

STDMETHODIMP CComSignedDoc::createNotaryInfoFromFile(IComSignatureInfo *pSigInfo, 
    BSTR szOcspRespFile, BSTR szNotaryCertFile, long *pRetError, IComNotaryInfo *pStoreTo)
{
  int errcode;
  USES_CONVERSION;
  NotaryInfo* pNotInfo = NULL;
  if(m_pSignedDoc)
	errcode = ::NotaryInfo_new_file(&pNotInfo, m_pSignedDoc, 
      ((CComSignatureInfo*)pSigInfo)->getSignatureInfo(), GetBSTR(szOcspRespFile), 
      GetBSTR(szNotaryCertFile));
  *pRetError = errcode;
  if (pNotInfo != (NotaryInfo*)0) {
    ((CComNotaryInfo*)pStoreTo)->shiftStructPointer(pNotInfo);
	((CComNotaryInfo*)pStoreTo)->setSignatureInfo(((CComSignatureInfo*)pSigInfo)->getSignatureInfo());
  }

	return S_OK;
}

//////////////////////////////////////////////////////////
//      DigiDoc file handling methods
//AA 2004/05/20 - ei saanud salvestada faili, kui kataloogis olid tppidega thed
STDMETHODIMP CComSignedDoc::createSignedDoc(BSTR szOutputFile, BSTR szOldFileName, long *pRetError)
{
	USES_CONVERSION;
	char *p1 = 0;

	if(m_pSignedDoc) {
        if(m_pSignedDoc && ::hasSignatureWithWrongDataFileHash(m_pSignedDoc)) {
            ddocDebug(1, "deleteSignatureInfo", "Cannot add signature in ddoc with invalid DataFile hashes!");
            *pRetError = ERR_FILE_WRITE;
            return S_OK;
        }
		p1 = GetBSTR(szOutputFile);
		ddocDebug(3, "CComSignedDoc::createSignedDoc", "Writing to: %s", p1);
		*pRetError = ::createSignedDoc(m_pSignedDoc, 
			(::SysStringLen(szOldFileName) > 0) ? GetBSTR(szOldFileName) : NULL, p1);
		ddocDebug(3, "CComSignedDoc::createSignedDoc", "Wrote");
		if(m_szDigiDocFile) {
			ddocDebug(3, "CComSignedDoc::createSignedDoc", "Free: %s", m_szDigiDocFile);
		
			free(m_szDigiDocFile);
			m_szDigiDocFile = 0;
		}
		ddocDebug(3, "CComSignedDoc::createSignedDoc", "Assign: %s", p1);
		
		m_szDigiDocFile = strdup(p1);
	}
	return S_OK;
}


STDMETHODIMP CComSignedDoc::readSignedDoc(BSTR szFileName, long checkFileDigest, long nMaxDFLen, long *pRetErr)
{
  SignedDoc* pSigDoc = 0;
  char *p1 = 0;

  USES_CONVERSION;
  p1 = GetBSTR(szFileName);
  // TODO: read bdoc
  *pRetErr = ddocSaxReadSignedDocFromFile(&pSigDoc, p1, checkFileDigest, nMaxDFLen);
  if (pSigDoc != (SignedDoc*)0) {
	if(m_szDigiDocFile) {
		free(m_szDigiDocFile);
		m_szDigiDocFile = 0;
	}
	m_szDigiDocFile = strdup(p1);
    if(m_pSignedDoc) {
      ::SignedDoc_free(m_pSignedDoc);
      m_pSignedDoc = 0;
    }
    shiftStructPointer(pSigDoc);
  }
  return S_OK;
}


//////////////////////////////////////////////////////////
//      verification methods
STDMETHODIMP CComSignedDoc::verifyDigest(IComSignatureInfo *pSigInfo, 
    IComDocInfo *pDocInfo, BSTR szDataFile, BSTR szFileName, long *nRetResult)
{
  USES_CONVERSION;

  if(m_pSignedDoc)
  *nRetResult = ::verifySigDocDigest(m_pSignedDoc, 
      ((CComSignatureInfo*)pSigInfo)->getSignatureInfo(),
	  ((CComDocInfo*)pDocInfo)->getDocInfo(), GetBSTR(szDataFile));

  return S_OK;
}

STDMETHODIMP CComSignedDoc::verifyMimeDigest(IComSignatureInfo *pSigInfo, 
    IComDocInfo *pDocInfo, BSTR szFileName, long *nRetResult)
{
  USES_CONVERSION;

  if(m_pSignedDoc)
  *nRetResult = ::verifySigDocMimeDigest(m_pSignedDoc, 
      ((CComSignatureInfo*)pSigInfo)->getSignatureInfo(),
      ((CComDocInfo*)pDocInfo)->getDocInfo(), GetBSTR(szFileName));

  return S_OK;
}

STDMETHODIMP CComSignedDoc::verifySignatureInfo(IComSignatureInfo *pSigInfo, 
    BSTR szDataFile, long *nRetResult)
{
	USES_CONVERSION;

	if(!m_pSignedDoc)
		return S_OK;
	*nRetResult = ::Digi_verifySignatureInfo(m_pSignedDoc, 
      ((CComSignatureInfo*)pSigInfo)->getSignatureInfo(),
      GetBSTR(szDataFile));
	// VS: ver 2.3.12 - consolidated verification in verifySignature method
	if(!(*nRetResult))
		*nRetResult = ::Digi_verifyNotaryInfoWithCertSearch(m_pSignedDoc, 
		((CComSignatureInfo*)pSigInfo)->getSignatureInfo()->pNotary);
  return S_OK;
}

STDMETHODIMP CComSignedDoc::verifySigDoc(BSTR szDataFile, long *nRetResult)
{
	unsigned int dfLen = ::SysStringLen(szDataFile);
	USES_CONVERSION;

	if(!m_pSignedDoc)
		return S_OK;
	*nRetResult = ::Digi_verifySigDocWithCertSearch(m_pSignedDoc,  
         ((dfLen > 0) ? GetBSTR(szDataFile) : NULL));

	return S_OK;
}

STDMETHODIMP CComSignedDoc::verifyNotaryInfo(IComNotaryInfo *pNotInfo, long *nRetResult)
{
	USES_CONVERSION;
	if(!m_pSignedDoc)
		return S_OK;
	// VS: ver 2.3.12 - consolidated verification in verifySignature method
	//*nRetResult = ::Digi_verifyNotaryInfoWithCertSearch(m_pSignedDoc, 
    //  ((CComNotaryInfo*)pNotInfo)->getNotaryInfo());
	*nRetResult = ERR_OK; // ?
	return S_OK;
}



//////////////////////////////////////////////////////////
//        OCSP methods

//////////////////////////////////////////////////////////
//           delete methods
STDMETHODIMP CComSignedDoc::deleteDataFile(BSTR szDataFileId, long *nRetCode)
{
	USES_CONVERSION;

	if(m_pSignedDoc)
		*nRetCode = ::DataFile_delete(m_pSignedDoc, GetBSTR(szDataFileId));

	return S_OK;
}

STDMETHODIMP CComSignedDoc::deleteNotaryInfo(BSTR szNotInfoId, long *nRetCode)
{
	USES_CONVERSION;
	NotaryInfo* pNotInfo = 0;

	if(m_pSignedDoc) {
		pNotInfo = ::getNotaryWithId(m_pSignedDoc, GetBSTR(szNotInfoId));
		if(pNotInfo) {
			SignatureInfo *pSigInfo = ::ddocGetSignatureForNotary(getSignedDoc(), pNotInfo);
		if(pSigInfo)
			*nRetCode = ::NotaryInfo_delete(pSigInfo);
		}
	}
	return S_OK;
}

STDMETHODIMP CComSignedDoc::deleteSignatureInfo(BSTR szSigInfoId, long *nRetCode)
{
	USES_CONVERSION;
	if(m_pSignedDoc) {
        if(m_pSignedDoc && ::hasSignatureWithWrongDataFileHash(m_pSignedDoc)) {
            ddocDebug(1, "deleteSignatureInfo", "Cannot add signature in ddoc with invalid DataFile hashes!");
            *nRetCode = ERR_FILE_WRITE;
            return S_OK;
        }
		*nRetCode = ::SignatureInfo_delete(getSignedDoc(), GetBSTR(szSigInfoId));
		if(m_szDigiDocFile) {
			*nRetCode = ::createSignedDoc(m_pSignedDoc, m_szDigiDocFile, m_szDigiDocFile);
		}
	}
	return S_OK;
}


//////////////////////////////////////////////////////////////
//        helpers
//////////////////////////////////////////////////////////////

CComSignedDoc::CComSignedDoc() {

	_CrtSetDbgFlag(
		  _CRTDBG_ALLOC_MEM_DF
		| _CRTDBG_CHECK_CRT_DF
		| _CRTDBG_CHECK_ALWAYS_DF
		| _CRTDBG_DELAY_FREE_MEM_DF
		| _CRTDBG_LEAK_CHECK_DF );


  m_pSignedDoc = NULL;
  //m_pDds = NULL;
  m_szDigiDocFile = NULL;
}

CComSignedDoc::~CComSignedDoc() {
  if(m_pSignedDoc)
	  SignedDoc_free(m_pSignedDoc);
  // TODO: remove file?
  if(m_szDigiDocFile)
	  free(m_szDigiDocFile);
}


SignedDoc* CComSignedDoc::getSignedDoc() { 
  return m_pSignedDoc;
}


void CComSignedDoc::shiftStructPointer(SignedDoc* pNew) {
  if(m_pSignedDoc)
	  SignedDoc_free(m_pSignedDoc);
  m_pSignedDoc = pNew;
}



STDMETHODIMP CComSignedDoc::getConfirmation(IComSignatureInfo* pSigInfo, BSTR pkcs12File, BSTR password, BSTR notaryURL, BSTR proxyHost, BSTR proxyPort, LONG* nRetCode)
{
	USES_CONVERSION;

	if(m_pSignedDoc)
		*nRetCode = ::Digi_getConfirmationWithCertSearch(m_pSignedDoc, 
			((CComSignatureInfo*)pSigInfo)->getSignatureInfo(), 
			GetBSTR(pkcs12File), GetBSTR(password), GetBSTR(notaryURL), 
			GetBSTR(proxyHost), GetBSTR(proxyPort));

	return S_OK;
}


STDMETHODIMP CComSignedDoc::getDataFileNameWidthId(BSTR szId, BSTR* pFileName)
{
	int iErr;
	char *psBuf = 0;
	int  iBufLen = 0;

	if(m_pSignedDoc) {
		iErr = ddocGetDataFileFilename(m_pSignedDoc, GetBSTR(szId), (void **)&psBuf, &iBufLen);
		if(psBuf)
			Convert::out_retval_sz2BSTR(psBuf, pFileName);
	}
	if(psBuf)
		freeLibMem(psBuf);
	return S_OK;
}
