/*

Wrap of DigiCoc library SignedDoc structure to provide COM-interface
Copyright (C) AS Sertifitseerimiskeskus

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

GNU Lesser General Public Licence is available at
http://www.gnu.org/copyleft/lesser.html

*/
// ComSignedDoc.h : Declaration of the CComSignedDoc

#ifndef __COMSIGNEDDOC_H_
#define __COMSIGNEDDOC_H_

#include "resource.h"       // main symbols

#include "libdigidoc/DigiDocLib.h"
#include "libdigidoc/DigiDocSAXParser.h"
#include "ComDataFile.h"
#include "ComSignatureInfo.h"
#include "ComNotaryInfo.h"
#include "ComDocInfo.h"
#include "ComCertSearchStore.h"
#include "ComCertSearch.h"

/////////////////////////////////////////////////////////////////////////////
// CComSignedDoc

class ATL_NO_VTABLE CComSignedDoc : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CComSignedDoc, &CLSID_ComSignedDoc>,
	public IDispatchImpl<IComSignedDoc, &IID_IComSignedDoc, &LIBID_DIGIDOCLIBCOMLib>
{
public:
	CComSignedDoc();
	~CComSignedDoc();

DECLARE_REGISTRY_RESOURCEID(IDR_COMSIGNEDDOC)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CComSignedDoc)
	COM_INTERFACE_ENTRY(IComSignedDoc)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

// IComSignedDoc
public:
	STDMETHOD(deleteSignatureInfo)(/*[in]*/ BSTR szSigInfoId, /*[out, retval]*/ long* nRetCode);
	STDMETHOD(deleteNotaryInfo)(/*[in]*/ BSTR szNotInfoId, /*[out, retval]*/ long* nRetCode);
	STDMETHOD(deleteDataFile)(/*[in]*/ BSTR szDataFileId, /*[out, retval]*/ long* nRetCode);
	STDMETHOD(verifyNotaryInfo)(/*[in]*/ IComNotaryInfo* pNotInfo, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(verifySigDoc)(/*[in]*/ BSTR szDataFile, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(verifySignatureInfo)(/*[in]*/ IComSignatureInfo* pSigInfo, /*[in]*/ BSTR szDataFile, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(verifyMimeDigest)(/*[in]*/ IComSignatureInfo* pSigInfo, /*[in]*/ IComDocInfo* pDocInfo, /*[in]*/ BSTR szFileName, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(verifyDigest)(/*[in]*/ IComSignatureInfo* pSigInfo, /*[in]*/ IComDocInfo* pDocInfo, /*[in]*/ BSTR szDataFile, /*[in]*/ BSTR szFileName, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(readSignedDoc)(/*[in]*/ BSTR szFileName, /*[in]*/ long checkFileDigest, /*in*/ long nMaxDFLen, /*[out, retval]*/ long* pRetErr);
	STDMETHOD(createSignedDoc)(/*[in]*/ BSTR szOutputFile, /*[in]*/ BSTR szOldFileName, /*[out, retval]*/ long* pRetError);
	STDMETHOD(createNotaryInfoFromFile)(/*[in]*/ IComSignatureInfo* pSigInfo, /*[in]*/ BSTR szOcspRespFile, /*[in]*/ BSTR szNotaryCertFile, /*[out]*/ long* pRetError, /*[in]*/ IComNotaryInfo* pStoreTo);
	STDMETHOD(createNotaryInfo)(/*[in]*/ IComSignatureInfo* pSigInfo, /*[in]*/ IComNotaryInfo* pStoreTo);
	STDMETHOD(getNotaryWithSigId)(/*[in]*/ BSTR szSigId, /*[in]*/ IComNotaryInfo* pStoreTo);
	STDMETHOD(getNotaryWithId)(/*[in]*/ BSTR szId, /*[in]*/ IComNotaryInfo* pStoreTo);
	STDMETHOD(getNotaryInfo)(/*[in]*/ long nIndex, /*[in]*/ IComNotaryInfo* pStoreTo);
	STDMETHOD(addAllDocInfos)(/*[in]*/ IComSignatureInfo* pSigInfo);
	STDMETHOD(getSignatureWithId)(/*[in]*/ BSTR szId, /*[in]*/ IComSignatureInfo* pStoreTo);
	STDMETHOD(getSignature)(/*[in]*/ long nIndex, /*[in]*/ IComSignatureInfo* pStoreTo);
	STDMETHOD(createSignatureInfo)(/*[in]*/ IComSignatureInfo* pStoreTo);
	STDMETHOD(calculateDataFileSizeAndDigest)(/*[in]*/ BSTR szId, /*[in]*/ BSTR szFileName, /*[in]*/ long digType, /*[out, retval]*/ long* nRetResult);
	STDMETHOD(initialize)(/*[in]*/ BSTR szFormat, /*[in]*/ BSTR szVersion);
	STDMETHOD(getDataFileWithId)(/*[in]*/ BSTR szId, /*[in]*/ IComDataFile* pStoreTo);
	STDMETHOD(getDataFile)(/*[in]*/ long nIndex, /*[in, out]*/ IComDataFile* pStoreTo);
	STDMETHOD(createDataFile)(/*[in]*/ BSTR szFileName, /*[in]*/ BSTR szContentType, /*[in]*/ BSTR szMimeType, /*[in]*/ long nSize, /*[in]*/ VARIANT baDigest, /*[in]*/ long nDigLen, /*[in]*/ BSTR szDigType, /*[in]*/ BSTR szCharset, /*[in]*/ IComDataFile* pStoreTo);
	STDMETHOD(get_nNotaries)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_nSignatures)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_nDataFiles)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_szFormatVer)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(get_szFormat)(/*[out, retval]*/ BSTR *pVal);


  SignedDoc* getSignedDoc();
  void shiftStructPointer(SignedDoc* pNew);


protected:
  SignedDoc* m_pSignedDoc;
  //DDS *m_pDds;
  char *m_szDigiDocFile;
  long lDdsSesscode;
public:
	STDMETHOD(getConfirmation)(IComSignatureInfo* pSigInfo, BSTR pkcs12File, BSTR password, BSTR notaryURL, BSTR proxyHost, BSTR proxyPort, LONG* nRetCode);
	STDMETHOD(getDataFileNameWidthId)(BSTR szId, BSTR* pFileName);
	
};

#endif //__COMSIGNEDDOC_H_
